include.set_path('apps');
include.namespace("mtango");
include.plugins(
    'lang/json',
    'controller',
    'model/jsonp'
);
include.resources();

include(function(){ //runs after prior includes are loaded
    include.models('DeviceProxy','Action','Request','Response','DeviceAttribute','DeviceCommand');
});;
include.set_path('jmvc/plugins/lang/json');
include.plugins('lang');
include('json');;
include.set_path('jmvc/plugins/lang');
include({path: 'standard_helpers.js', shrink_variables: false});

	;
include.set_path('jmvc/plugins/lang');
// Several of the methods in this plugin use code adapated from Prototype
//  Prototype JavaScript framework, version 1.6.0.1
//  (c) 2005-2007 Sam Stephenson

MVC.String = {};
MVC.String.strip = function(string){
	return string.replace(/^\s+/, '').replace(/\s+$/, '');
};


MVC.Function = {};
MVC.Function.params = function(func){
	var ps = func.toString().match(/^[\s\(]*function[^(]*\((.*?)\)/)[1].split(",");
	if( ps.length == 1 && !ps[0]) return [];
	for(var i = 0; i < ps.length; i++) ps[i] = MVC.String.strip(ps[i]);
	return ps;
};

/**
 * @class MVC.Native
 */
MVC.Native ={};
MVC.Native.
/**
 * 
 * @param {Object} class_name
 * @param {Object} source
 */
extend = function(class_name, source){
	if(!MVC[class_name]) MVC[class_name] = {};
	var dest = MVC[class_name];
	for (var property in source){
		dest[property] = source[property];
		if(!MVC._no_conflict){
			window[class_name][property] = source[property];
			if(typeof source[property] == 'function'){
				var names = MVC.Function.params(source[property]);
    			if( names.length == 0) continue;
				//var first_arg = names[0];
				//if( first_arg.match(class_name.substr(0,1).toLowerCase()  ) || (first_arg == 'func' && class_name == 'Function' )  ){
					MVC.Native.set_prototype(class_name, property, source[property]);
				//}
			}
		}
	}
};
MVC.Native.set_prototype = function(class_name, property_name, func){
	if(!func) func = MVC[class_name][property_name];
    window[class_name].prototype[property_name] = function(){
		var args = [this];
		for (var i = 0, length = arguments.length; i < length; i++) args.push(arguments[i]);
		return func.apply(this,args  );
	};
};

/**
 * @class MVC.Native.Object
 * @alias MVC.Object
 * Object helpers
 */
MVC.Native.Object = {};

MVC.Native.Object.
/**
 * Copies one object to another
 * @param {Object} destination
 * @param {Object} source
 * @return {Object} the destination object
 */
extend = function(destination, source) {
  for (var property in source)
    destination[property] = source[property];
  return destination;
};

MVC.Native.Object.
/**
 * For an object, returns url params.
 * @param {Object} object
 * @param {Object} name
 */
to_query_string = function(object,name){
	if(typeof object != 'object') return object;
	return MVC.Native.Object.to_query_string.worker(object,name).join('&');
};
MVC.Native.Object.to_query_string.worker = function(obj,name){
	var parts2 = [];
	for(var thing in obj){
		if(obj.hasOwnProperty(thing)) {
			var value = obj[thing];
            if(value && value.constructor === Date){
                
                value =  value.getUTCFullYear()+'-'+
                    MVC.Number.to_padded_string(value.getUTCMonth() + 1,2) + '-' +
                    MVC.Number.to_padded_string(value.getUTCDate(),2) + ' ' +
                    MVC.Number.to_padded_string(value.getUTCHours(),2) + ':' +
                    MVC.Number.to_padded_string(value.getUTCMinutes(),2) + ':' +
                    MVC.Number.to_padded_string(value.getUTCSeconds(),2);
            }
			if(value instanceof Array && value.length){
				var newer_name = encodeURIComponent(name ? name+'['+thing+']' : thing) ;
                for(var i = 0; i < value.length; i++){
                    var nice_val = encodeURIComponent(value[i].toString());
                    parts2.push( newer_name+'='+nice_val )  ;
                }
            } else if (value && typeof value != 'object') {
				var nice_val = encodeURIComponent(value.toString());
				var newer_name = encodeURIComponent(name ? name+'['+thing+']' : thing) ;
				parts2.push( newer_name+'='+nice_val )  ;
			}else{
				parts2 = parts2.concat( MVC.Native.Object.to_query_string.worker(value,  name ? name+'['+thing+']' : thing ))
			}
		}
	}
	return parts2;
};

/* 
 * @class MVC.Native.String
 * @alias MVC.String
 * When not in no-conflict mode, JMVC adds the following helpers to string
 */
MVC.Native.extend('String', 
/* @Static*/
{
    /*
     * Capitalizes a string
     * @param {String} s the string to be lowercased.
     * @return {String} a string with the first character capitalized, and everything else lowercased
     */
	capitalize : function(s) {
		return s.charAt(0).toUpperCase()+s.substr(1).toLowerCase();
	},
    /**
     * Returns if a string has another string inside it.
     * @param {String} string String that is being scanned
     * @param {String} pattern String that we are looking for
     * @return {Boolean} true if the string has pattern, false if otherwise
     */
	include : function(s, pattern){
		return s.indexOf(pattern) > -1;
	},
    /**
     * Returns if string ends with another string
     * @param {String} s String that is being scanned
     * @param {String} pattern What the string might end with
     * @return {Boolean} true if the string ends wtih pattern, false if otherwise
     */
	ends_with : function(s, pattern) {
	    var d = s.length - pattern.length;
	    return d >= 0 && s.lastIndexOf(pattern) === d;
	},
    /**
     * Capitalizes a string from something undercored. Examples:
     * @code_start
     * MVC.String.camelize("one_two") //-> "oneTwo"
     * "three-four".camelize() //-> threeFour
     * @code_end
     * @param {String} s
     * @return {String} a the camelized string
     */
	camelize: function(s){
		var parts = s.split(/_|-/);
		for(var i = 1; i < parts.length; i++)
			parts[i] = MVC.String.capitalize(parts[i]);
		return parts.join('');
	},
    /**
     * Like camelize, but the first part is also capitalized
     * @param {String} s
     * @return {String}
     */
	classize: function(s){
		var parts = s.split(/_|-/);
		for(var i = 0; i < parts.length; i++)
			parts[i] = MVC.String.capitalize(parts[i]);
		return parts.join('');
	},
    /*
     * @function strip
     * @param {String} s returns a string with leading and trailing whitespace removed.
     */
	strip : MVC.String.strip
});

//Date Helpers, probably should be moved into its own class

/* 
 * @class MVC.Native.Array
 * @alias MVC.Array
 * When not in no-conflict mode, JMVC adds the following helpers to array
 */
MVC.Native.extend('Array',
/* @static*/
{ 
	/**
	 * Searchs an array for item.  Returns if item is in it.
	 * @param {Object} array
	 * @param {Object} item an item that is matched with ==
	 * @return {Boolean}
	 */
    include: function(a, item){
		for(var i=0; i< a.length; i++){
			if(a[i] == item) return true;
		}
		return false;
	},
    /**
     *
     * @param {Function|Object} el filter function or object to remove
     * @returns {Array}
     */
    remove : function(a,el){
    var result = [];
    for(var i = 0, size = a.length; i < size; ++i){
        if(typeof(el) == 'function' && el(a[i])){
            continue;
        } else if(a[i] == el){
            continue;
        } else {
            result.push(a[i]);
        }
    }
    return result;
    }
});
MVC.Array.
    /**
     * Creates an array from another object.  Typically, this is used to give arguments array like properties.
     * @param {Object} iterable an array like object with a length property.
     * @return {Array}
     */
	from= function(iterable){
		 if (!iterable) return [];
		var results = [];
	    for (var i = 0, length = iterable.length; i < length; i++)
	      results.push(iterable[i]);
	    return results;
	}
MVC.Array.
    /**
     * Returns if the object is an array
     * @param {Object} array a possible array object
     * @return {Boolean}
     */
    is = function(array){
    return Object.prototype.toString.call(array) === '[object Array]';
    }
MVC.Array.last = function () {
    if (this.length == 0)
        throw "Array: querying zero length array";
    if (this.length == 1)
        return this[0];
    return this[this.length - 1];
};

MVC.Array.first = function () {
    if (this.length == 0)
        throw "Array: querying zero length array";
    return this[0];
};

MVC.Array.findFirst = function (condition) {
    if (this.length == 0)
        throw "Array: querying zero length array";
    else if(this.length == 1)
        return this[0];
    else
        for(var i = 0; i<this.length;++i){
            if(condition(this[i],i))
                return this[i];
        }
    return this.last();
};
/**
 * Performs binary search of the find in this Array
 *
 * @param {*} find
 * @param {Function} comparator :=(el, ndx, find)
 * @return {*} element or null
 */
MVC.Array.binarySearch = function (find, comparator) {
    var low = 0, high = this.length - 1,
        i, comparison;
    while (low <= high) {
        i = Math.floor((low + high) / 2);
        comparison = comparator(this[i], i, find);
        if (comparison < 0) {
            low = i + 1;
            continue;
        }
        if (comparison > 0) {
            high = i - 1;
            continue;
        }
        return this[i];
    }
    return null;
};
/* 
 * @class MVC.Native.Function
 * @alias MVC.Function
 * When not in no-conflict mode, JMVC adds the following helpers to function
 */
MVC.Native.extend('Function', 
/* @static*/
{
	/**
	 * Binds a function to another object.  The object the function is binding
	 * to is the second argument.  Additional params are added to the callback function.
	 * @code_start
	 * //basic example
	 * var callback1 = MVC.Function.bind(function(){alert(this.library)}, {library: "jmvc"});
	 * //shows with prepended args
	 * var callback2 = MVC.Function.bind(
	 *     function(version, os){
	 *         alert(this.library+", "+version+", "+os);
	 *     },
	 *     {library: "jmvc"},
	 *     "1.5")
	 * @code_end
	 * @param {Function} f The function that is being bound.
	 * @param {Object} obj The object you want to bind to.
	 * @return {Function} the wrapping function.
	 */
    bind: function(f, obj) {
	  var args = MVC.Array.from(arguments);
	  args.shift();args.shift();
	  var __method = f, object = arguments[1];
	  return function() {
	    return __method.apply(object, args.concat(MVC.Array.from(arguments) )  );
	  }
	},
	params: MVC.Function.params
});
/* 
 * @class MVC.Native.Number
 * @alias MVC.Number
 * When not in no-conflict mode, JMVC adds the following helpers to number
 */
MVC.Native.extend('Number', 
/* @static*/
{
    /**
     * Changes a number to a string, but includes preceeding zeros.
     * @param {Object} number the number to be converted
     * @param {Object} length the number of zeros
     * @param {optional:Object} radix the numeric base (defaults to base 10);
     * @return {String} 
     */
    to_padded_string: function(n, len, radix) {
        var string = n.toString(radix || 10);
        var ret = '', needed = len - string.length;
        
        for(var i = 0 ; i < needed; i++) 
            ret += '0';
        return ret + string;
    }
});

MVC.Native.Array = MVC.Array;
MVC.Native.Function = MVC.Function;
MVC.Native.Number = MVC.Number;
MVC.Native.String = MVC.String;
MVC.Object = MVC.Native.Object;
if(!MVC._no_conflict){
    Array.from = MVC.Array.from;
    Array.last = MVC.Array.last;
    Array.first = MVC.Array.first;
    Array.findFirst = MVC.Array.findFirst;
    Array.binarySearch = MVC.Array.binarySearch;
};
include.set_path('jmvc/plugins/lang/json');
//Heavily borrowed from prototype.org

/**
 *  @add  MVC.Native.Number  Static
 */
MVC.Object.extend(MVC.Number, {
    /**
     * @plugin lang/json
     * @param {Object} number
     */
    to_json: function(number) {
        return isFinite(number) ? number.toString() : 'null';
    }
});

/**
 *  @add  MVC.Native.Date  Static
 */
if(!MVC.Date) MVC.Date = {};
MVC.Object.extend(MVC.Date, {
    /**
     * @plugin lang/json
     * @param {Object} date
     */
    to_json: function(date) {
      return '"' + date.getUTCFullYear() + '-' +
        MVC.Number.to_padded_string(date.getUTCMonth() + 1, 2) + '-' +
        MVC.Number.to_padded_string(date.getUTCDate(), 2)+ 'T' +
        MVC.Number.to_padded_string(date.getUTCHours(), 2) + ':' +
        MVC.Number.to_padded_string(date.getUTCMinutes(), 2) + ':' +
        MVC.Number.to_padded_string(date.getUTCSeconds(), 2) + 'Z"';
    }
});
/**
 *  @add  MVC.Native.String  Static
 */
MVC.Object.extend(MVC.String, {
    /**
     * @plugin lang/json
     * @param {Object} string
     */
    to_json: function(string) {
        var specialChar= {
            '\b': '\\b',
            '\t': '\\t',
            '\n': '\\n',
            '\f': '\\f',
            '\r': '\\r',
            '\\': '\\\\'
        };
        
        var escapedString = string.replace(/[\x00-\x1f\\]/g, function(match) {
          var character = specialChar[match[0]];
          return character ? character : '\\u00' + MVC.Number.to_padded_string(match[0].charCodeAt(), 2, 16);
        });
        return '"' + escapedString.replace(/"/g, '\\"') + '"';
    }
});
/**
 *  @add  MVC.Native.Array  Static
 */
MVC.Object.extend(MVC.Array,{
    /**
     * @plugin lang/json
     * @param {Object} array
     * @param {Object} nested
     */
    to_json: function(array,nested) {
        var results = [];
        for(var i=0; i < array.length; i++){
            var val =  MVC.Object.to_json(array[i],true);
            if(typeof val != 'undefined') results.push(val);
        }
        return '[' + results.join(', ') + ']';
    }
});
/**
 *  @add  MVC.Native.Object  Static
 */
MVC.Object.
/**
 * @plugin lang/json
 * @param {Object} object
 * @param {Object} nested
 */
to_json = function(object, nested){
    var type = typeof object;
    switch (type) {
      case 'undefined':
      case 'function':
      case 'unknown': return;
      case 'boolean': return object.toString();
      case 'string' : return MVC.String.to_json(object);
      case 'number' : return MVC.Number.to_json(object);
    }

    if (object === null) return 'null';
    
    
    switch (object.constructor){
        case Array : return MVC.Array.to_json(object);
        case Date : return MVC.Array.to_json(object);
    }
    
    if (object.to_json) return object.to_json();
    if (object.nodeType == 1) return;

    var results = [];
    for (var property in object) {
      var value = MVC.Object.to_json(object[property], true);
      if (typeof value != 'undefined')
        results.push(MVC.String.to_json(property) + ': ' + value);
    }
    return '{' + results.join(', ') + '}';
};

MVC.Object
/**
 * @param [String] json
 */
    .from_json = function(json){
    "use strict";

    if(typeof JSON != 'undefined') return JSON.parse(json);
    else return eval("var obj = " + json.replace('\n',' ') + "; obj");
};;
include.set_path('jmvc/plugins/controller');
include.plugins('lang','lang/inflector','dom/event','lang/class','lang/openajax','dom/data');
include('delegator','controller');
if(MVC.View) include.plugins('controller/view');
//if(include.get_env() == 'test') include('test');
include.set_path('jmvc/plugins/lang/inflector');
include.plugins('lang');
include('inflector');;
include.set_path('jmvc/plugins/lang/inflector');
// based on the Inflector class found on a DZone snippet contributed by Todd Sayre
// http://snippets.dzone.com/posts/show/3205

MVC.Inflector = {
  Inflections: {
    plural: [
    [/(quiz)$/i,               "$1zes"  ],
    [/^(ox)$/i,                "$1en"   ],
    [/([m|l])ouse$/i,          "$1ice"  ],
    [/(matr|vert|ind)ix|ex$/i, "$1ices" ],
    [/(x|ch|ss|sh)$/i,         "$1es"   ],
    [/([^aeiouy]|qu)y$/i,      "$1ies"  ],
    [/(hive)$/i,               "$1s"    ],
    [/(?:([^f])fe|([lr])f)$/i, "$1$2ves"],
    [/sis$/i,                  "ses"    ],
    [/([ti])um$/i,             "$1a"    ],
    [/(buffal|tomat)o$/i,      "$1oes"  ],
    [/(bu)s$/i,                "$1ses"  ],
    [/(alias|status)$/i,       "$1es"   ],
    [/(octop|vir)us$/i,        "$1i"    ],
    [/(ax|test)is$/i,          "$1es"   ],
    [/s$/i,                    "s"      ],
    [/$/,                      "s"      ]
    ],
    singular: [
    [/(quiz)zes$/i,                                                    "$1"     ],
    [/(matr)ices$/i,                                                   "$1ix"   ],
    [/(vert|ind)ices$/i,                                               "$1ex"   ],
    [/^(ox)en/i,                                                       "$1"     ],
    [/(alias|status)es$/i,                                             "$1"     ],
    [/(octop|vir)i$/i,                                                 "$1us"   ],
    [/(cris|ax|test)es$/i,                                             "$1is"   ],
    [/(shoe)s$/i,                                                      "$1"     ],
    [/(o)es$/i,                                                        "$1"     ],
    [/(bus)es$/i,                                                      "$1"     ],
    [/([m|l])ice$/i,                                                   "$1ouse" ],
    [/(x|ch|ss|sh)es$/i,                                               "$1"     ],
    [/(m)ovies$/i,                                                     "$1ovie" ],
    [/(s)eries$/i,                                                     "$1eries"],
    [/([^aeiouy]|qu)ies$/i,                                            "$1y"    ],
    [/([lr])ves$/i,                                                    "$1f"    ],
    [/(tive)s$/i,                                                      "$1"     ],
    [/(hive)s$/i,                                                      "$1"     ],
    [/([^f])ves$/i,                                                    "$1fe"   ],
    [/(^analy)ses$/i,                                                  "$1sis"  ],
    [/((a)naly|(b)a|(d)iagno|(p)arenthe|(p)rogno|(s)ynop|(t)he)ses$/i, "$1$2sis"],
    [/([ti])a$/i,                                                      "$1um"   ],
    [/(n)ews$/i,                                                       "$1ews"  ],
    [/s$/i,                                                            ""       ]
    ],
    irregular: [
    ['move',   'moves'   ],
    ['sex',    'sexes'   ],
    ['child',  'children'],
    ['man',    'men'     ],
    ['foreman', 'foremen'],
    ['person', 'people'  ]
    ],
    uncountable: [
    "sheep",
    "fish",
    "series",
    "species",
    "money",
    "rice",
    "information",
    "equipment"
    ]
  },
  pluralize: function(word) {
    for (var i = 0; i < MVC.Inflector.Inflections.uncountable.length; i++) {
      var uncountable = MVC.Inflector.Inflections.uncountable[i];
      if (word.toLowerCase() == uncountable) {
        return uncountable;
      }
    }
    for (var i = 0; i < MVC.Inflector.Inflections.irregular.length; i++) {
      var singular = MVC.Inflector.Inflections.irregular[i][0];
      var plural   = MVC.Inflector.Inflections.irregular[i][1];
      if ((word.toLowerCase() == singular) || (word == plural)) {
        return word.substring(0,1)+plural.substring(1);
      }
    }
    for (var i = 0; i < MVC.Inflector.Inflections.plural.length; i++) {
      var regex          = MVC.Inflector.Inflections.plural[i][0];
      var replace_string = MVC.Inflector.Inflections.plural[i][1];
      if (regex.test(word)) {
        return word.replace(regex, replace_string);
      }
    }
  },
  singularize: function(word) {
    for (var i = 0; i < MVC.Inflector.Inflections.uncountable.length; i++) {
      var uncountable = MVC.Inflector.Inflections.uncountable[i];
      if (word.toLowerCase() == uncountable) {
        return uncountable;
      }
    }
    for (var i = 0; i < MVC.Inflector.Inflections.irregular.length; i++) {
      var singular = MVC.Inflector.Inflections.irregular[i][0];
      var plural   = MVC.Inflector.Inflections.irregular[i][1];
      if ((word.toLowerCase() == singular) || (word.toLowerCase() == plural)) {
        return word.substring(0,1)+singular.substring(1);
      }
    }
    for (var i = 0; i < MVC.Inflector.Inflections.singular.length; i++) {
      var regex          = MVC.Inflector.Inflections.singular[i][0];
      var replace_string = MVC.Inflector.Inflections.singular[i][1];
      if (regex.test(word)) {
        return word.replace(regex, replace_string);
      }
    }
  }
};
/**
 *  @add  MVC.Native.String  Static
 */
MVC.Native.extend('String', {
  /**
   * Pluralizes a string
   * @plugin lang/inflector
   * @param {String} string string to be pluralized
   * @param {optional:Number} count
   * @param {optional:String} plural
   * @return {String}
   */
  pluralize: function(string, count, plural) {
    if (typeof count == 'undefined') {
      return MVC.Inflector.pluralize(string);
    } else {
      return count + ' ' + (1 == parseInt(count) ? string : plural || MVC.Inflector.pluralize(string));
    }
  },
  /**
   * Returns the singular version of a string
   * @plugin lang/inflector
   * @param {String} string the string to be singularized
   * @param {optional:Number} count
   * @return {String}
   */
  singularize: function(string, count) {
    if (typeof count == 'undefined') {
      return MVC.Inflector.singularize(string);
    } else {
      return count + " " + MVC.Inflector.singularize(string);
    }
  },
  /**
   * Returns if the string is singular
   * @plugin lang/inflector
   * @param {String} string
   * @return {Boolean}
   */
  is_singular: function(string){
    if(MVC.String.singularize(string) == null && string)
        return true;
    return false;
  }
});;
include.set_path('jmvc/plugins/dom/event');
/**
 * Event describes 2 functions
 * 	Event.observe
 * 	Event.stopObserving
 */

if(typeof Prototype == 'undefined') 
	include("standard");
else{
	include("prototype_event");
}
	
//jQuery's wont work for controllers because it doesn't allow capture;
include.set_path('jmvc/plugins/dom/event');
// The code from the event plugin comes from 
// JavaScript: the Definitive Guide by David Flanagan
// Copyright 2006 O'Reilly Media

if(document.addEventListener) {
	MVC.Event = {
		observe: function(el, eventType, handler, capture) {
			if(capture == null) capture = false; 
			el.addEventListener(eventType, handler, capture);
    	},
		stop_observing : function(el, eventType, handler, capture) {
	        if(capture == null) capture = false;
	        el.removeEventListener(eventType, handler, false);
	    }
	};
}else if(document.attachEvent) {
/*
 * @class MVC.Event
 * JavaScriptMVC's default Event functionality. This functionality should rarely be used. 
 * In place of registering event handlers directly, you are HIGHLY encouraged to 
 * create Controllers to handle event registration and callback in a very dry fassion. 
 * If you use other libraries like prototype or jQuery, their functionailty will be mapped to these functions.
 * 
 * 
 * <h3>Example</h3>
The following calls checkForm on the form element with id 'signinForm' being submitted.
@code_start
MVC.Event.observe(MVC.$E('signinForm'), 'submit', checkForm);
@code_end
 * 
 */
  MVC.Event=
 /* @Static*/ 
  {
	/**
	 * Registers an event handler on a DOM element.
	 * @param {Object} element
	 * @param {Object} eventType
	 * @param {optional:Object} handler defaults to false.
	 */
    observe: function(element, eventType, handler) {
        if (MVC.Event._find(element, eventType, handler) != -1) return;
        var wrappedHandler = function(e) {
            if (!e) e = window.event;
            
            
            
            var event = {
                _event: e, 
                type: e.type, 
                target: e.srcElement,  
                currentTarget: element, 
                relatedTarget: eventType == 'mouseover' ?e.fromElement : e.toElement, //mouseout gets toElement
                eventPhase: (e.srcElement==element)?2:3,
                clientX: e.clientX, clientY: e.clientY,
                screenX: e.screenX, screenY: e.screenY,
                altKey: e.altKey, ctrlKey: e.ctrlKey,
                shiftKey: e.shiftKey, charCode: e.keyCode,
                stopPropagation: function() {this._event.cancelBubble = true;},
                preventDefault: function() {this._event.returnValue = false;},
                which: e.which || (e.button & 1 ? 1 : ( e.button & 2 ? 3 : ( e.button & 4 ? 2 : 0 ) ))
            };

            
            
            if (Function.prototype.call) 
                handler.call(element, event);
            else {
                element._currentHandler = handler;
                element._currentHandler(event);
                element._currentHandler = null;
            }
        };
        element.attachEvent("on" + eventType, wrappedHandler);
        var h = {
            element: element,
            eventType: eventType,
            handler: handler,
            wrappedHandler: wrappedHandler
        };
        var d = element.document || element, w = d.parentWindow, id = MVC.Event._uid(); 
        if (!w._allHandlers) w._allHandlers = {}; 
        w._allHandlers[id] = h;
        if (!element._handlers) element._handlers = [];
        element._handlers.push(id);
        if (!w._onunloadHandlerRegistered) {
            w._onunloadHandlerRegistered = true;
            w.attachEvent("onunload", MVC.Event._removeAllHandlers);
        }
    },
    /**
     * Unregisters an event handler.
     * @param {Object} element
     * @param {Object} eventType
     * @param {Object} handler
     */
	stop_observing: function(element, eventType, handler) {
        var i = MVC.Event._find(element, eventType, handler);
        if (i == -1) return; 
        var d = element.document || element, w = d.parentWindow, handlerId = element._handlers[i], h = w._allHandlers[handlerId];
        element.detachEvent("on" + eventType, h.wrappedHandler);
        element._handlers.splice(i, 1);
        delete w._allHandlers[handlerId];
    },
	_find: function(element, eventType, handler) {
        var handlers = element._handlers;
        if (!handlers) return -1;
        var d = element.document || element, w = d.parentWindow;
        for(var i = handlers.length-1; i >= 0; i--) {
            var h = w._allHandlers[handlers[i]];
            if(h.eventType == eventType && h.handler == handler)  return i;
        }
        return -1;
    },
	_removeAllHandlers: function() {
        var w = this;
        for(var id in w._allHandlers) {
            if(! w._allHandlers.hasOwnProperty(id) ) continue;
			var h = w._allHandlers[id]; 
            if(h.element) h.element.detachEvent("on" + h.eventType, h.wrappedHandler);
            delete w._allHandlers[id];
        }
    },
	_counter : 0,
	_uid : function() { return "h" + MVC.Event._counter++; }
  };
};

if(!MVC._no_conflict && typeof Event == 'undefined'){
	Event = MVC.Event;
};
include.set_path('jmvc/plugins/lang/class');
//MVC.Class 
// This is a modified version of John Resig's class
// It provides class level inheritence and callbacks.

(function(){
  var initializing = false, fnTest = /xyz/.test(function(){xyz;}) ? /\b_super\b/ : /.*/;
  // The base Class implementation (does nothing)
  
  /**
   * @constructor MVC.Class
   * Class provides simple simulated inheritance in JavaScript. 
   * It is based off John Resig's [http://ejohn.org/blog/simple-javascript-inheritance/|Simple Class] 
   * Inheritance library.  Besides prototypal inheritance, it adds a few important features:
   * <ul>
   *     <li>Static inheritance</li>
   *     <li>Class initialization callbacks</li>
   *     <li>Introspection</li>
   * </ul>
   * <h2>Examples</h2>
   * <h3>Basic example</h3>
   * Creates a class with a className (used for introspection), static, and prototype members:
   * @code_start
   * Monster = MVC.Class.extend('monster',
   * /* @static *|
   * {
   *   count: 0
   * },
   * /* @prototype *|
   * {
   *   init : function(name){
   *     this.name = name;
   *     this.Class.count++
   *   }
   * })
   * hydra = new Monster('hydra')
   * dragon = new Monster('dragon')
   * hydra.name        // -> hydra
   * Monster.count     // -> 2
   * Monster.className // -> 'monster'
   * @code_end
   * Notice that the prototype init function is called when a new instance of Monster is created.
   * <h3>Static property inheritance</h3>
   * Demonstrates inheriting a class poperty.
   * @code_start
   * First = MVC.Class.extend(
   * {
   *     static_method : function(){ return 1;}
   * },{})
   * Second = First.extend({
   *     static_method : function(){ return this._super()+1;}
   * },{})
   * Second.static_method() // -> 2
   * @code_end
   * <h3 id='introspection'>Introspection</h3>
   * Often, it's nice to create classes whose name helps determine functionality.  Ruby on
   * Rails's [http://api.rubyonrails.org/classes/ActiveRecord/Base.html|ActiveRecord] ORM class 
   * is a great example of this.  Unfortunately, JavaScript doesn't have a way of determining
   * an object's name, so the developer must provide a name.
   * 
   * For example, Documentation's [MVC.Doc.Directive|directives] use their className to be added to
   * different [MVC.Doc.Pair|comment-code pairs] in the appropriate way.  Just by defining:
   * @code_start
   * MVC.Doc.Directive.Author = MVC.Class.extend('author');
   * @code_end
   * you tell the documentation engine to look for @author and whatever comes after it as this.author.
   * className is saved as a static property.  You can access it from instance methods like:
   * @code_start
   * this.Class.className
   * @code_end
   * <h3>Construtors</h3>
   * Class uses static and class initialization constructor functions.  
   * @code_start
   * MyClass = MVC.Class.extend(
   * {
   *   init: function(){} //static constructor
   * },
   * {
   *   init: function(){} //prototype constructor
   * })
   * @code_end
   * The static init constructor is called after
   * a class has been created, but before [MVC.Class.static.extended|extended] is called on its base class.  
   * This is a good place to add introspection and similar class setup code.
   * 
   * The prototype callback is called whenever a new instance of the class is created.
   * 
   * 
   * @init Creating a new instance of an object that has extended MVC.Class 
        calls the init prototype function and returns a new instance of the class.
   * 
   */
  
  MVC.Class = function(){};
  // Create a new Class that inherits from the current class.
  /* @Static*/
  MVC.Class.
    /**
     * Extends a class with new static and prototype functions.  There are a variety of ways
     * to use extend:
     * @code_start
     * //with className, static and prototype functions
     * MVC.class.extend('task',{ STATIC },{ PROTOTYPE })
     * //with just static and prototype functions
     * MVC.class.extend({ STATIC },{ PROTOTYPE })
     * //with just classname and prototype functions
     * MVC.class.extend('task',{ PROTOTYPE })
     * //with just prototype functions
     * MVC.class.extend({ PROTOTYPE })
     * //With just a className
     * MVC.class.extend('task')
     * @code_end
     * @param {optional:String} className the classes name (used for classes w/ introspection)
     * @param {optional:Object} klass the new classes static/class functions
     * @param {optional:Object} proto the new classes prototype functions
     * @return {MVC.Class} returns the new class
     */
    extend = function(className, klass, proto) {
    if(typeof className != 'string'){
        proto = klass;
        klass = className;
        className = null;
    }
    if(!proto){
        proto = klass;
        klass = null;
    }
    proto = proto || {};
    var _super_class = this;
    var _super = this.prototype;
    // Instantiate a base class (but only create the instance,
    // don't run the init constructor)
    initializing = true;
    var prototype = new this();
    initializing = false;
    // Copy the properties over onto the new prototype
    for (var name in proto) {
      // Check if we're overwriting an existing function
      prototype[name] = typeof proto[name] == "function" &&
        typeof _super[name] == "function" && fnTest.test(proto[name]) ?
        (function(name, fn){
          return function() {
            var tmp = this._super;
           
            // Add a new ._super() method that is the same method
            // but on the super-class
            this._super = _super[name];
           
            // The method only need to be bound temporarily, so we
            // remove it when we're done executing
            var ret = fn.apply(this, arguments);       
            this._super = tmp;
           
            return ret;
          };
        })(name, proto[name]) :
        proto[name];
    }
    // The dummy class constructor
    function Class() {
      // All construction is actually done in the init method
      if ( !initializing && this.init )
        this.init.apply(this, arguments);
    }
    // Populate our constructed prototype object
    Class.prototype = prototype;
    
    Class.prototype.Class = Class;
    // Enforce the constructor to be what we expect
    Class.constructor = Class;
    // And make this class extendable
    
    for(var name in this){
        if(this.hasOwnProperty(name) && name != 'prototype'){
            Class[name] = this[name];
        }
    }
    
    for (var name in klass) {
      Class[name] = typeof klass[name] == "function" &&
        typeof Class[name] == "function" && fnTest.test(klass[name]) ?
        (function(name, fn){
          return function() {
            var tmp = this._super;
            this._super = _super_class[name];
            var ret = fn.apply(this, arguments);       
            this._super = tmp;
            return ret;
          };
        })(name, klass[name]) :
        klass[name];
	};
    Class.extend = arguments.callee;
    /**
     * @attribute className 
     * The name of the class provided for [#introspection|introspection] purposes.
     */
    if(className) Class.className = className;
    /*
     * @function init
     * Called when a new Class is created
     * @param {MVC.Class} class the new class
     */
    if(Class.init) Class.init(Class);
    /*
     * @function extended
     * Called with whatever classes extend your class
     * @param {MVC.Class} Class the extending class.
     */
    if(_super_class.extended) _super_class.extended(Class);
    /* @Prototype*/
    return Class;
    /* @function init
     * Called with the same arguments as new Class(arguments ...) when a new instance is created.
     */
    //Breaks up code
    /**
     * @attribute Class
     * Access to the static properties of the instance's class.
     */
  };
})();

if(!MVC._no_conflict && typeof Class == 'undefined'){
	Class = MVC.Class;
};
include.set_path('jmvc/plugins/lang/openajax');
/*******************************************************************************
 * OpenAjax.js
 *
 * Reference implementation of the OpenAjax Hub, as specified by OpenAjax Alliance.
 * Specification is under development at: 
 *
 *   http://www.openajax.org/member/wiki/OpenAjax_Hub_Specification
 *
 * Copyright 2006-2008 OpenAjax Alliance
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not 
 * use this file except in compliance with the License. You may obtain a copy 
 * of the License at http://www.apache.org/licenses/LICENSE-2.0 . Unless 
 * required by applicable law or agreed to in writing, software distributed 
 * under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
 * CONDITIONS OF ANY KIND, either express or implied. See the License for the 
 * specific language governing permissions and limitations under the License.
 *
 ******************************************************************************/

// prevent re-definition of the OpenAjax object
if(!window["OpenAjax"]){
	/**
	 * @class OpenAjax
	 * Use OpenAjax.hub to publish and subscribe to messages.
	 */
    OpenAjax = new function(){
		var t = true;
		var f = false;
		var g = window;
		var ooh = "org.openajax.hub.";

		var h = {};
		this.hub = h;
		h.implementer = "http://openajax.org";
		h.implVersion = "1.0";
		h.specVersion = "1.0";
		h.implExtraData = {};
		var libs = {};
		h.libraries = libs;

		h.registerLibrary = function(prefix, nsURL, version, extra){
			libs[prefix] = {
				prefix: prefix,
				namespaceURI: nsURL,
				version: version,
				extraData: extra 
			};
			this.publish(ooh+"registerLibrary", libs[prefix]);
		}
		h.unregisterLibrary = function(prefix){
			this.publish(ooh+"unregisterLibrary", libs[prefix]);
			delete libs[prefix];
		}

		h._subscriptions = { c:{}, s:[] };
		h._cleanup = [];
		h._subIndex = 0;
		h._pubDepth = 0;

		h.subscribe = function(name, callback, scope, subscriberData, filter)			
		{
			if(!scope){
				scope = window;
			}
			var handle = name + "." + this._subIndex;
			var sub = { scope: scope, cb: callback, fcb: filter, data: subscriberData, sid: this._subIndex++, hdl: handle };
			var path = name.split(".");
	 		this._subscribe(this._subscriptions, path, 0, sub);
			return handle;
		}

		h.publish = function(name, message)		
		{
			var path = name.split(".");
			this._pubDepth++;
			this._publish(this._subscriptions, path, 0, name, message);
			this._pubDepth--;
			if((this._cleanup.length > 0) && (this._pubDepth == 0)) {
				for(var i = 0; i < this._cleanup.length; i++) 
					this.unsubscribe(this._cleanup[i].hdl);
				delete(this._cleanup);
				this._cleanup = [];
			}
		}

		h.unsubscribe = function(sub) 
		{
			var path = sub.split(".");
			var sid = path.pop();
			this._unsubscribe(this._subscriptions, path, 0, sid);
		}
		
		h._subscribe = function(tree, path, index, sub) 
		{
			var token = path[index];
			if(index == path.length) 	
				tree.s.push(sub);
			else { 
				if(typeof tree.c == "undefined")
					 tree.c = {};
				if(typeof tree.c[token] == "undefined") {
					tree.c[token] = { c: {}, s: [] }; 
					this._subscribe(tree.c[token], path, index + 1, sub);
				}
				else 
					this._subscribe( tree.c[token], path, index + 1, sub);
			}
		}

		h._publish = function(tree, path, index, name, msg, pcb, pcid) {
			if(typeof tree != "undefined") {
				var node;
				if(index == path.length) {
					node = tree;
				} else {
					this._publish(tree.c[path[index]], path, index + 1, name, msg, pcb, pcid);
					this._publish(tree.c["*"], path, index + 1, name, msg, pcb, pcid);			
					node = tree.c["**"];
				}
				if(typeof node != "undefined") {
					var callbacks = node.s;
					var max = callbacks.length;
					for(var i = 0; i < max; i++) {
						if(callbacks[i].cb) {
							var sc = callbacks[i].scope;
							var cb = callbacks[i].cb;
							var fcb = callbacks[i].fcb;
							var d = callbacks[i].data;
							var sid = callbacks[i].sid;
							var scid = callbacks[i].cid;
							if(typeof cb == "string"){
								// get a function object
								cb = sc[cb];
							}
							if(typeof fcb == "string"){
								// get a function object
								fcb = sc[fcb];
							}
							if((!fcb) || (fcb.call(sc, name, msg, d))) {
							  if((!pcb) || (pcb(name, msg, pcid, scid))) {
								  cb.call(sc, name, msg, d, sid);
							  }
							}
						}
					}
				}
			}
		}
			
		h._unsubscribe = function(tree, path, index, sid) {
			if(typeof tree != "undefined") {
				if(index < path.length) {
					var childNode = tree.c[path[index]];
					this._unsubscribe(childNode, path, index + 1, sid);
					if(childNode.s.length == 0) {
						for(var x in childNode.c) 
					 		return;		
						delete tree.c[path[index]];	
					}
					return;
				}
				else {
					var callbacks = tree.s;
					var max = callbacks.length;
					for(var i = 0; i < max; i++) 
						if(sid == callbacks[i].sid) {
							if(this._pubDepth > 0) {
								callbacks[i].cb = null;	
								this._cleanup.push(callbacks[i]);						
							}
							else
								callbacks.splice(i, 1);
							return; 	
						}
				}
			}
		}
		// The following function is provided for automatic testing purposes.
		// It is not expected to be deployed in run-time OpenAjax Hub implementations.
		h.reinit = function()
		{
			for (var lib in OpenAjax.hub.libraries) {
				delete OpenAjax.hub.libraries[lib];
			}
			OpenAjax.hub.registerLibrary("OpenAjax", "http://openajax.org/hub", "1.0", {});

			delete OpenAjax._subscriptions;
			OpenAjax._subscriptions = {c:{},s:[]};
			delete OpenAjax._cleanup;
			OpenAjax._cleanup = [];
			OpenAjax._subIndex = 0;
			OpenAjax._pubDepth = 0;
		}
	};
	// Register the OpenAjax Hub itself as a library.
	OpenAjax.hub.registerLibrary("OpenAjax", "http://openajax.org/hub", "1.0", {});

}
OpenAjax.hub.registerLibrary("JavaScriptMVC", "http://JavaScriptMVC.com", "1.5", {});;
include.set_path('jmvc/plugins/dom/data');
//from jQuery


MVC.Dom ={
    data: function( elem, name, data ) {
		elem = elem == window ?
			windowData :
			elem;
        var cache = elem.__mvc;


		if ( !cache )
			elem.__mvc = {};

		if ( data !== undefined )
			elem.__mvc[ name ] = data;

		// Return the named cache data, or the ID for the element
		return name ?
			elem.__mvc[ name ] :
			elem.__mvc;
	},
	remove_data: function( elem, name ) {
		elem = elem == window ?
			windowData :
			elem;

		var cache = elem.__mvc;

		// If we want to remove a specific section of the element's data
		if ( name ) {
			if ( cache ) {
				// Remove the section of cache data
				delete cache[ name ];

				// If we've removed all the data, remove the element's cache
				name = "";

				for ( name in cache )
					break;

				if ( !name )
					MVC.Dom.remove_data( elem );
			}

		// Otherwise, we want to remove all of the element's data
		} else {
			// Clean up the element expando
			try {
				delete elem.__mvc;
			} catch(e){
				// IE has trouble directly removing the expando
				// but it's ok with using removeAttribute
				if ( elem.removeAttribute )
					elem.removeAttribute( "__jmvc" );
			}
		}
	}
};
include.set_path('jmvc/plugins/controller');
/**
 * @constructor
 * Attaches listeners for delegated events.
 * @init Creates a new delegator listener
 * @param {String} selector a css selector
 * @param {String} event a dom event
 * @param {Function} f a function to call
 */
MVC.Delegator = function(selector, event, f, element){
    this._event = event;
    this._selector = selector
    this._func = f;
    this.element = element || document.documentElement;
    MVC.Delegator.jmvc(this.element)
    if(event == 'contextmenu' && MVC.Browser.Opera) return this.context_for_opera();
    if(event == 'submit' && MVC.Browser.IE) return this.submit_for_ie();
	if(event == 'change' && MVC.Browser.IE) return this.change_for_ie();
	if(event == 'change' && MVC.Browser.WebKit) return this.change_for_webkit();
	
    this.add_to_delegator();
};

MVC.Object.extend(MVC.Delegator,
/* @Static*/
{
    /**
     * Adds _jmvc to elements to keep track of delegation events.
     * @param {Object} element
     * @return {Object} the jmvc object.
     */
    jmvc : function(element){
        var data = MVC.Dom.data(element);
        if(!data.delegation_events) data.delegation_events = {};
        if(data.responding == null) data.responding = true;
        return data;
    },
    /**
     * Adds kill() on an event.
     * @param {Object} event
     */
    add_kill_event: function(event){ //this should really be in event
		if(!event.kill){
			if(!event) event = window.event;
            var killed = false;
			event.kill = function(){
				killed = true;
			    try{
				    if (event.stopPropagation)  event.stopPropagation(); 
				    if (event.preventDefault)  event.preventDefault();
			    }catch(e){}
			};
			event.is_killed = function(){return killed;};
            event.stop_propagation = function(){
                killed = true;
                try{
			        if (event.stopPropagation)  event.stopPropagation(); 
			    }catch(e){}
            }
            event.prevent_default = function(){
                try{
			        if (event.preventDefault)  event.preventDefault(); 
			    }catch(e){}
            }
		}	
	},
    /**
     * Used for sorting events on an object
     * @param {Object} a
     * @param {Object} b
     * @return {Number} -1,0,1 depending on how a and b should be sorted.
     */
    sort_by_order: function(a,b){
    	if(a.order < b.order) return 1;
    	if(b.order < a.order) return -1;
    	var ae = a._event, be = b._event;
    	if(ae == 'click' &&  be == 'change') return 1;
    	if(be == 'click' &&  ae == 'change') return -1;
    	return 0;
    },
    /**
     * Stores all delegated events
     */
    events: {},
    onload_called : false
})
MVC.Event.observe(window, 'load', function(){
    MVC.Delegator.onload_called = true;
})
/* @Prototype*/
MVC.Delegator.prototype = {
    /*
     * returns the event that should actually be used.  In practice, this is just used to switch focus/blur
     * to activate/deactivate for ie.
     * @return {String} the adjusted event name.
     */
    event: function(){
    	if(MVC.Browser.IE){
            if(this._event == 'focus')
    			return 'activate';
    		else if(this._event == 'blur')
    			return 'deactivate';
    	}
    	return this._event;
    },
    /*
     * Returns if capture should be used (blur and focus)
     * @return {Boolean} true for focus / blur, false if otherwise
     */
    capture: function(){
        return MVC.Array.include(['focus','blur'],this._event);
    },
    /**
     * If there are no special cases, this is called to add to the delegator.
     * @param {String} selector - css selector
     * @param {String} event - event selector
     * @param {Function} func - a function that will be called
     */
    add_to_delegator: function(selector, event, func){
        var s = selector || this._selector;
        var e = event || this.event();
        var f = func || this._func;
        var delegation_events = MVC.Dom.data(this.element,"delegation_events");
        if(!delegation_events[e] || delegation_events[e].length == 0){
            var bind_function = MVC.Function.bind(this.dispatch_event, this)
            MVC.Event.observe(this.element, e, bind_function, this.capture() );
            delegation_events[e] = [];
            delegation_events[e]._bind_function = bind_function;
		}
		delegation_events[e].push(this);
    },
    _remove_from_delegator : function(event_type){
        var event = event_type || this.event();
        var events = MVC.Dom.data(this.element,"delegation_events")[event];
        for(var i = 0; i < events.length;i++ ){
            if(events[i] == this){
                events.splice(i, 1);
                break;
            }
        }
        if(events.length == 0){
            MVC.Event.stop_observing(this.element, event, events._bind_function, this.capture() );
        }
    },
    /*
     * Handles the submit case for IE.  It checks if a keypress return happens in an
     * input area or a submit button is clicked.
     */
    submit_for_ie : function(){
		this.add_to_delegator(null, 'click');
        this.add_to_delegator(null, 'keypress');
        
        this.filters= {
			click : function(el, event, parents){
				//check you are in a form
                if(el.nodeName.toUpperCase() == 'INPUT' && el.type.toLowerCase() == 'submit'){
                    for(var e = 0; e< parents.length ; e++) if(parents[e].tag == 'FORM') return true;
                }
                return false;
                
			},
			keypress : function(el, event, parents){
				if(el.nodeName.toUpperCase()!= 'INPUT') return false;
				var res = typeof Prototype != 'undefined' ? (event.keyCode == 13) : (event.charCode == 13)
                if(res){
                    for(var e = 0; e< parents.length ; e++) if(parents[e].tag == 'FORM') return true;
                }
                return false;
			}
		};
	},
    /*
     * Handles change events for IE.
     */
	change_for_ie : function(){
		this.add_to_delegator(null, 'click');
        this.add_to_delegator(null, 'keyup');
        this.add_to_delegator(null, 'beforeactivate')
        //return if right or not
        this.end_filters= {
			click : function(el, event){
				switch(el.nodeName.toLowerCase()){
                    case "select":
                        if(typeof el.selectedIndex == 'undefined') return false;
                        var data = MVC.Dom.data(el);
                        if( data._change_old_value == null){
        					data._change_old_value = el.selectedIndex.toString();
        					return false;
        				}else{
        					if(data._change_old_value == el.selectedIndex.toString()) return false;
        					data._change_old_value = el.selectedIndex.toString();
        					return true;
        				}
                        break;
                     case "input":
                         if(el.type.toLowerCase() =="checkbox" ) return true;
                         return false;
                     
                }
                return false;
			},
            keyup : function(el, event){
                if(el.nodeName.toLowerCase() != "select") return false;
                if(typeof el.selectedIndex == 'undefined') return false;
                var data = MVC.Dom.data(el);
                if( data._change_old_value == null){
                    data._change_old_value = el.selectedIndex.toString();
					return false;
				}else{
					if(data._change_old_value == el.selectedIndex.toString()){
                        return false;
                    }
					data._change_old_value = el.selectedIndex.toString();
					return true;
				}
            },
            beforeactivate : function(el, event){
                //we should probably check that onload has been called.
                return el.nodeName.toLowerCase() == 'input' 
                    && el.type.toLowerCase() =="radio" 
                    && !el.checked
                    && MVC.Delegator.onload_called  //IE selects this on the start
            }
		};
	},
    /*
     * Handles a change event for Safari.
     */
	change_for_webkit : function(){
		this.add_to_delegator(null, 'change');
		this.end_filters= {
			change : function(el, event){
                if(el.nodeName.toLowerCase() == 'input') return true;
				if(typeof el.value == 'undefined') return false; //sometimes it won't exist yet
				var old = el.getAttribute('_old_value');
				el.setAttribute('_old_value', el.value);
				return el.value != old;
			}
		};
	},
    /**
     * Handles a right click for Opera.  It looks for clicks with shiftkey pressed.
     */
    context_for_opera : function(){
        this.add_to_delegator(null, 'click');
        this.end_filters= {
			click : function(el, event){
				return event.shiftKey;
			}
        }
    },
    regexp_patterns:  {tag :    		/^\s*(\*|[\w\-]+)(\b|$)?/,
        				id :            /^#([\w\-\*]+)(\b|$)/,
    					className :     /^\.([\w\-\*]+)(\b|$)/},
    /*
     * returns and caches the select order for the css patern.
     * @retun {Array} array of objects that are used to match with the node_path
     */
    selector_order : function(){
		if(this.order) return this.order;
		var selector_parts = this._selector.split(/\s+/);
		var patterns = this.regexp_patterns;
		var order = [];
        if(this._selector)
		for(var i =0; i< selector_parts.length; i++){
			var v = {}, r, p =selector_parts[i];
			for(var attr in patterns){
				if( patterns.hasOwnProperty(attr) ){
					if( (r = p.match(patterns[attr]))  ) {
						if(attr == 'tag')
							v[attr] = r[1].toUpperCase();
						else
							v[attr] = r[1];
						p = p.replace(r[0],'');
					}
				}
			}
			order.push(v);
		}
		this.order = order;
		return this.order;
	},
    /**
     * Tests if an event matches an element.
     * @param {Object} el the element we are testing
     * @param {Object} event the event
     * @param {Object} parents an array of node order objects for the element
     * @return {Object} returns an object with node, order, and delegation_event attributes.
     */
    match: function(el, event, parents){
        if(this.filters && !this.filters[event.type](el, event, parents)) return null;
		//if(this.controller.className != 'main' &&  (el == document.documentElement || el==document.body) ) return false;
		var matching = 0;
		var selector_order = this.selector_order();
        if(selector_order.length == 0){ //attached to top node
            return {node: parents[0].element, order: 0, delegation_event: this}
        } 
        for(var n=0; n < parents.length; n++){
			var node = parents[n], match = selector_order[matching], matched = true;
			for(var attr in match){
				if(!match.hasOwnProperty(attr) || attr == 'element') continue;
				if(match[attr] && attr == 'className'){
					if(! MVC.Array.include(node.className.split(' '),match[attr])) matched = false;
				}else if(match[attr] && node[attr] != match[attr]){
					matched = false;
				}
			}
			if(matched){
				matching++;
                if(matching >= selector_order.length) {
                    if(this.end_filters && !this.end_filters[event.type](el, event)) return null;
                    return {node: node.element, order: n, delegation_event: this};
                }
			}
		}
		return null;
    },
    /**
     * Goes through the delegated events for the given event type (e.g. Click).  Orders the matches
     * by how nested they are in the dom.  Adds the kill function on the event, then dispatches each
     * event.  If kill is called, it will stop dispatching other events.
     * @param {Event} event the DOM event returned by a normal event handler.
     */
	dispatch_event: function(event){
        var target = event.target, matched = false, ret_value = true,matches = [];
		var delegation_events = MVC.Dom.data(this.element,"delegation_events")[event.type];
        var parents_path = this.node_path(target);
		for(var i =0; i < delegation_events.length;  i++){
			var delegation_event = delegation_events[i];
			var match_result = delegation_event.match(target, event, parents_path);
			if(match_result){
				matches.push(match_result);
			}
		}

		if(matches.length == 0) return true;
		MVC.Delegator.add_kill_event(event);
		matches.sort(MVC.Delegator.sort_by_order);
        var match;
		for(var m = 0; m < matches.length; m++){
            match = matches[m];
            ret_value = match.delegation_event._func( {event: event, element: MVC.$E(match.node), delegate: this.element} ) && ret_value;
			if(event.is_killed()) return false;
		}
	},
    /**
     * Returns an array of objects that represent the path of the node to documentElement.  Each item in the array
     * has a tag, className, id, and element attribute.
     * @param {Object} el element in the dom that is nested under the documentElement
     * @return {Array} representation of the path between the element and the DocumentElement
     */
    node_path: function(el){
        var body = this.element,parents = [],iterator =el;
		if(iterator == body) return [{tag: iterator.nodeName, className: iterator.className, id: iterator.id, element: iterator}]
        do{
            parents.unshift({tag: iterator.nodeName, className: iterator.className, id: iterator.id, element: iterator});
        }while(((iterator = iterator.parentNode) != body )&& iterator)
        if(iterator)
            parents.unshift({tag: iterator.nodeName, className: iterator.className, id: iterator.id, element: iterator});
        return parents;
	},
    destroy : function(){
        //remove from events
        if(this._event == 'contextmenu' && MVC.Browser.Opera){
            return this._remove_from_delegator("click");
        }
        if(this._event == 'submit' && MVC.Browser.IE) {
            this._remove_from_delegator("keypress");
            return this._remove_from_delegator("click");
        }
    	if(this._event == 'change' && MVC.Browser.IE){
            this._remove_from_delegator("keyup");
            this._remove_from_delegator("beforeactivate");
            return this._remove_from_delegator("click");
        } 
    	//if(this._event == 'change' && MVC.Browser.WebKit){
        //    return this._remove_from_delegator();
        //}
        this._remove_from_delegator()
    }
};;
include.set_path('jmvc/plugins/controller');
// submitted by kangax
MVC.Object.is_number = function(o){
    return o &&(  typeof o == 'number' || ( typeof o == 'string' && !isNaN(o) ) );
};

/* 
 * Controllers respond to events. If something happens in your application, be it a click, or
 * a [MVC.Model Model] being updated, a controller should respond to it.
 * <h3>Example</h3>
@code_start
//Instead of:
$('.tasks').click(function(e){ ... })
//do this
TasksController = MVC.Controller.extend('tasks',{
  click: function(params){...}
})
@code_end
 * <h2>Actions</h2>
 * To respond to events, controllers simply name their event handling functions to match
 * an [MVC.Controller.Action Action].  
 *
 * In the previous example, TasksController's click action is matched by the [MVC.Controller.Action.Event Event] Action. 
 * Event matches functions that are combination of CSS selector and event name.  Here's another example:
@code_start
TasksController = MVC.Controller.extend('tasks',{
  ".delete mouseover": function(params){ ... }
})
@code_end
 * <h3>Types of Actions</h3>
 * There are many types of Actions.  By default, Controller will match [MVC.Controller.Action.Event Event] and
 * [MVC.Controller.Action.Subscribe Subscribe] actions.  To match other actions, include their plugins.
 * 
<table>
	<tr>
        <th>Action</th><th>Format</th><th>Description</th>
    </tr>
    <tbody  style="font-size: 10px;">
    <tr>
        <td>[MVC.Controller.Action.Event Event]</td>
        <td>[CSS] [change|click|...]</td>
        <td>Matches standard DOM events</td>
    </tr>
    <tr>
        <td>[MVC.Controller.Action.Subscribe Subscribe]</td>
        <td>[OpenAjax.hub event] subscribe</td>
        <td>Subscribes this action to OpenAjax hub.</td>
    </tr>
    <tr>
        <td>[MVC.Controller.Action.Drag Drag]</td>
        <td>[CSS] [dragstart|dragging|...]</td>
        <td>Matches events on a dragged object</td>
    </tr>
    <tr>
        <td>[MVC.Controller.Action.Drop Drop]</td>
        <td>[CSS] [dropadd|dropover|...]</td>
        <td>Matches events on a droppable object</td>
    </tr>
    <tr>
        <td>[MVC.Controller.Action.EnterLeave EnterLeave]</td>
        <td>[CSS] [mouseenter|mouseleave.]</td>
        <td>Similar to mouseover, mouseout, but handles nested elements.</td>
    </tr>
    <tr>
        <td>[MVC.Controller.Action.Hover Hover]</td>
        <td>[CSS] [hoverenter|hoverleave.]</td>
        <td>Similar to mouseenter, but only gets called if the user stops on an element.</td>
    </tr>
    <tr>
        <td>[MVC.Controller.Action.Lasso Lasso]</td>
        <td>[CSS] [lassostart|...]</td>
        <td>Allows you to lasso elements.</td>
    </tr>
    <tr>
        <td>[MVC.Controller.Action.Selectable Selectable]</td>
        <td>[CSS] [selectadd|...]</td>
        <td>Matches events on elements that can be selected by the lasso.</td>
    </tr>
    </tbody>
</table>
 * 
 * <h2>Naming Controllers</h2>
 * Controllers use their name to limit the DOM they act upon.  Depending if the controller name is 
 * plural, singular or main, it changes which elements it responds to.
 * <h3>Singular Controllers</h3>
 * Singluar controllers respond to events in or on an element with an id that matches the controller name.
@code_start
//matches &lt;div id="file_manager"&gt;&lt;/div&gt;
FileManagerController = MVC.Controller.extend('file_manager')
@code_end
 * <h3>Plural Controllers</h3>
 * Plural controllers respond to events in or on elements with classNames that match the singular 
 * controller name.
@code_start
//matches &lt;div class="task"&gt;&lt;/div&gt;
TasksController = MVC.Controller.extend('tasks')
@code_end
 * If you want to match events on an element with the id, add '#' to the start of your action.  For example:
@code_start
TasksController = MVC.Controller.extend('tasks',{
  click : function(){ .. }     //matches &lt;div class="task"&gt;&lt;/div&gt;
  "# click" : function(){ .. } //matches &lt;div id="tasks"&gt;&lt;/div&gt;
})
@code_end
 * <h3>Main Controllers</h3>
 * Controllers with the name 'main' can match events anywhere in the DOM.  
 * 
 * <h2>Params</h2>
 * Controller actions get called with an instance of [MVC.Controller.Params].  Params
 * provide aditional functionality based on the param data.  Killing events is a good example.
 * Some actions get called with classes that inherit from MVC.Controller.Params.
 * Check your action's params for the data that gets passed to your event handling functions.
 */
MVC.Controller = MVC.Class.extend(
/* @Static*/
{
    /*
     * Looks for controller actions and hooks them up to delegator
     */
    init: function(){
        if(!this.className) return;
        this.singularName =  MVC.String.singularize(this.className);
        if(!MVC.Controller.controllers[this.className]) MVC.Controller.controllers[this.className] = [];
        MVC.Controller.controllers[this.className].unshift(this);
        var val, act;
        
        if(!this.modelName)
            this.modelName = MVC.String.is_singular(this.className) ? this.className : MVC.String.singularize(this.className)
        if(this._should_attach_actions)
            this._create_actions();
        //load tests
        if(include.get_env() == 'test'){
            var path = MVC.root.join('test/functional/'+this.className+'_controller_test.js');
    		
    		var exists = include.check_exists(path);
    		if (exists) {
				MVC.Console.log('Loading: "test/functional/' + this.className + '_controller_test.js"');
                include('../test/functional/'+this.className+'_controller_test.js');
			}
			else {
				MVC.Console.log('Test Controller not found at "test/functional/' + this.className + '_controller_test.js"');
			}
        }
        this._path =  include.get_path().match(/(.*?)controllers/)[1]+"controllers";
        this.action_name = 'init';
    },
    _should_attach_actions : true,
    _create_actions : function(){
        this.actions = {};
        for(var action_name in this.prototype){
    		val = this.prototype[action_name];
    		if( typeof val == 'function' && action_name != 'Class'){
                for(var a = 0 ; a < MVC.Controller.actions.length; a++){
                    act = MVC.Controller.actions[a];
                    if(act.matches(action_name)){
                        var callback = this.dispatch_closure(action_name);
                        this.actions[action_name] =new act(action_name, callback, this.className, this._element, this._events);
                    }
                }
            }
	    }
    },
    dispatch_closure: function(f_name){
        return MVC.Function.bind(function(params){
            params = params || {};
            params.action = f_name;
            params.controller = this;
            params = params.constructor == MVC.Controller.Params ? params : new MVC.Controller.Params(params)
			return this.dispatch(f_name,  params );
		},this);
    },
    /**
     * Calls the Controller prototype function specified by action_name with the given params.
     * @param {String} action_name The name of the action to be called.
     * @param {Controller.Params} params The params the action will be called with.
     */
    dispatch: function(action_name, params){
		if(!action_name) action_name = 'index';
		
		if(typeof action_name == 'string'){
			if(!(action_name in this.prototype) ) throw 'No action named '+action_name+' was found for '+this.Class.className+' controller.';
		}else{ //action passed TODO:  WHERE IS THIS USED?
			action_name = action_name.name;
		}
        var instance = this._get_instance(action_name , params);
		return this._dispatch_action(instance,action_name, params );
	},
    _get_instance : function(action_name,  params){
          return new this(action_name, params);
    },
	_dispatch_action: function(instance, action_name, params){
        if(!this._listening) return;
        instance.params = params;
		instance.action_name = action_name;
        return instance[action_name](params);
	},
    controllers : {},
    actions: [],
    publish: function(message, params){
        //var subscribers = MVC.Controller.Action.Subscribe.events[message];
        //if(!subscribers) return;
        //for(var i =0 ; i < subscribers.length; i++){
        //    subscribers[i](params);
        //}
        OpenAjax.hub.publish(message, params);
    },
    get_controller_with_name_and_action: function(controller_name, action) {
        var controllers = MVC.Controller.controllers[controller_name];
        if(!controllers) return null;
		for(var i = 0; i < controllers.length; i++) {
            var controller = controllers[i];
            if (controller.prototype[action]) return controller;
        }
        return null;
     },
     /**
      * The name of the model this controller can uses for param functions like element_instance
      */
     modelName: null,
     /**
      * A flag if controllers can respond to events.
      */
     _listening : true,
     _events : MVC.Delegator.events,
     _element : document.documentElement
},
/* @Prototype*/
{
    /*
     * Returns a function that when called, calls the action with parameters passed to the function. 
     * This is very useful for creating callbacks for Ajax functionality. 
     * The callback is called on the same controller instance that created the callback. 
     * This allows you to easily pass objects between request and response without resorting to closures. 
     * Example:
@code_start
Controller('todos',{
   "a click" : function(params){ 
      this.element = params.element;
	  this.element.innerHTML = 'deleting ...';
	  new Ajax('delete', {onComplete: this.continue_to('deleted')}
   },
   deleted : function(response){
      this.element.parentNode.removeChild(this.element);
   }
});
@code_end
     * @param {String} action Name of prototype function you want called
     * @return {Function} function that when called, directs to another controller function
     */
    continue_to :function(action){
		var args = MVC.Array.from(arguments)
        var action = args.shift();
		if(typeof this[action] != 'function'){ throw 'There is no action named '+action+'. ';}
		return MVC.Function.bind(function(){
			this.action_name = action;
			this[action].apply(this, args.concat(MVC.Array.from(arguments)));
		}, this);
	},
    /**
     * Calls an action after some delay
     * @param {Object} delay
     * @param {Object} action_name
     * @param {Object} params
     */
    delay: function(delay, action_name, params){
		if(typeof this[action_name] != 'function'){ throw 'There is no action named '+action_name+'. ';}
		
        return setTimeout(MVC.Function.bind(function(){
			this.Class._dispatch_action(this, action_name ,  params )
		}, this), delay );
    },
    /**
     * Publishes a message to OpenAjax.hub.  Other controllers 
     * @param {String} message
     * @param {Object} data
     */
    publish: function(message, data){
        this.Class.publish(message,data);
    }
});


/*
 * MVC.Controller.Action is and abstract base class.
 * Controller Action classes are used to match controller prototype functions. 
 * Inheriting classes must provide a static matches function.
 * 
 * When a new controller is created, it iterates through its prototype functions and tests
 * each action if it matches.  If there is a match, the controller creates a new action.
 * 
 * The action is responsible to callback the function when appropriate.  It typically uses
 * dispatch_closure to call functions appropriately.  
 */
MVC.Controller.Action = MVC.Class.extend(
/* @Static */
{
    /**
     * If the class has provided a matches function, adds this class to the list of 
     * controller actions.
     */
    init: function(){
        if(this.matches) MVC.Controller.actions.push(this);
    }
},
/* @Prototype */
{
    /**
     * Called with prototype functions that match this action.
     * @param {String} action_name
     * @param {Function} f
     * @param {MVC.Controller} controller
     */
    init: function(action_name, callback, className, element){
        this.action = action_name;
        this.callback = callback;
        this.className = className;
        this.element = element
    },
    /**
     * Disables an action.
     */
    destroy: function(){
        
    }
});
/**
 * Subscribes to an OpenAjax.hub event.
 * <h3>Example</h3>
@code_start javascript
TasksController = MVC.Controller.extend('tasks',
{
  "task.create subscribe" : function(params){
     var published_data = params.data; //published data always in params.data
  }
});
@code_end
 */
MVC.Controller.Action.Subscribe = MVC.Controller.Action.extend(
/* @Static*/
{
    
    match: new RegExp("(.*?)\\s?(subscribe)$"),
    /**
     * Matches "(.*?)\\s?(subscribe)$"
     * @param {String} action_name
     */
    matches: function(action_name){
        return this.match.exec(action_name);
    }
},
/* @Prototype*/
{
    /**
     * @param {Object} action
     * @param {Object} f
     * @param {Object} controller
     */
    init: function(action_name, callback, className, element){
        this._super(action_name, callback, className, element);
        this.message();
        this.subscription = OpenAjax.hub.subscribe(this.message_name, MVC.Function.bind(this.subscribe, this) );
    },
    /**
     * Gets the message name from the action name.
     */
    message: function(){
        this.parts = this.action.match(this.Class.match);
        this.message_name = this.parts[1];
    },
    subscribe : function(event_name, data){
        var params = data || {};
        params.event_name = event_name
        this.callback(params)
    },
    destroy : function(){
        OpenAjax.hub.unsubscribe(this.subscription)
        this._super();
    }
})
/*
 * Default event delegation based actions
 */
MVC.Controller.Action.Event = MVC.Controller.Action.extend(
/* @Static*/
{
    /**
     * Matches change, click, contextmenu, dblclick, keydown, keyup, keypress, mousedown, mousemove, mouseout, mouseover, mouseup, reset, resize, scroll, select, submit, dblclick, focus, blur, load, unload
     * @param {Object} action_name
     */
    match: new RegExp("^(?:(.*?)\\s)?(change|click|contextmenu|dblclick|keydown|keyup|keypress|mousedown|mousemove|mouseout|mouseover|mouseup|reset|resize|scroll|select|submit|dblclick|focus|blur|load|unload)$"),
    /*
     * Matches change, click, contextmenu, dblclick, keydown, keyup, keypress, mousedown, mousemove, 
     * mouseout, mouseover, mouseup, reset, resize, scroll, select, submit, dblclick, 
     * focus, blur, load, unload
     * @return {Boolean} true if a prototype function name matches an action.
     */
    matches: function(action_name){
        return this.match.exec(action_name);
    }
},
/* @Prototype*/
{    
    init: function(action_name, callback, className, element){
        this._super(action_name, callback, className, element);
        this.css_and_event();
        
        var selector = this.selector();
        if(selector != null){
            this.delegator = new MVC.Delegator(selector, this.event_type, callback, element );
        }
    },
    /*
     * Splits the action name into its css and event parts.
     */
    css_and_event: function(){
        this.parts = this.action.match(this.Class.match);
        this.css = this.parts[1] || "";
        this.event_type = this.parts[2];
    },
    /*
     * Deals with main controller specific delegation (blur and focus)
     */
    main_controller: function(){
	    if(!this.css && MVC.Array.include(['blur','focus'],this.event_type)){
            MVC.Event.observe(window, this.event_type, MVC.Function.bind(function(event){
                this.callback({event: event, element: window})
            }, this))
            return;
        }
        return this.css;
    },
    /*
     * Handles a plural controller name
     * @return {String} the css with the controller name included
     */
    plural_selector : function(){
		if(this.css == "#" || this.css.substring(0,2) == "# "){
			var newer_action_name = this.css.substring(2,this.css.length)
            if(this.element == document.documentElement){
                return '#'+this.className + (newer_action_name ?  ' '+newer_action_name : '') ;
            }else{
                return (newer_action_name ?  ' '+newer_action_name : '') ;
            }
		}else{
            //if(this.element == document.documentElement){
			    return '.'+MVC.String.singularize(this.className)+(this.css? ' '+this.css : '' );
            //}else{
            //    return this.css;
            //}
		}
	},
    /*
     * Handles a singular controller name
     * @return {String} the css with the controller name included
     */
    singular_selector : function(){
        if(this.element == document.documentElement)
            return '#'+this.className+(this.css? ' '+this.css : '' );
        else
            return this.css;
    },
    /*
     * Gets the full css selector for this action
     * @return {String/null} returns a string css if Delegator should be used, null if otherwise.
     */
    selector : function(){
        if(MVC.Array.include(['load','unload','resize','scroll'],this.event_type)){
            MVC.Event.observe(window, this.event_type, MVC.Function.bind(function(event){
                this.callback({event: event, element: window})
            }, this));
            return;
        }
        //if(!this.className){
        //    this.css_selector = this.css
        //}else 
        if(this.className == 'main') 
            this.css_selector = this.main_controller();
        else
            this.css_selector = MVC.String.is_singular(this.className) ? 
                this.singular_selector() : this.plural_selector();
        return this.css_selector;
    },
    destroy : function(){
        if(this.delegator) this.delegator.destroy();
        this._super();
    }
});

/* @Constructor
 * Instances of Controller.Params are passed to Event based actions.
 * 
 * <h3>Example</h3>
@code_start
MVC.Controller.extend('todos', {
   mouseover : function(params){ 
      <span class="magic">params</span>.element.style.backgroundColor = 'Red';
   },
   mouseout : function(params){
      <span class="magic">params</span>.element.style.backgroundColor = '';
      <span class="magic">params</span>.event.stop();
   },
   "img click" : function(params){
   	  <span class="magic">params</span>.class_element().parentNode.removeSibiling(params.class_element());
   }
})
@code_end
 * @init Creates a new Controller.Params object.
 * @param {Object} params An object you want to pass to a controller
 */

MVC.Controller.Params = function(params){
	var params = params || {};
    var killed = false;
	this.kill = function(){
		killed = true;
        if(params.event && params.event.kill) params.event.kill();
	};
	this.is_killed = function(){return params.event.is_killed ?  params.event.is_killed() :  killed ;};
    
    for(var thing in params){
		if( params.hasOwnProperty(thing) ) this[thing] = params[thing];
	}
    this.constructor = MVC.Controller.Params;
};

/* @Prototype*/
MVC.Controller.Params.prototype = {
	/*
	 * Returns data in a hash for a form.
	 * @return {Object} Nested form data.
	 */
    form_params : function(){
		var data = {};
		if(this.element.nodeName.toLowerCase() != 'form') return data;
		var els = this.element.elements, uri_params = [];
		for(var i=0; i < els.length; i++){
			var el = els[i];
			if(el.type.toLowerCase()=='submit') continue;
			var key = el.name || el.id, key_components = key.match(/(\w+)/g), value;
            if(!key) continue;     
			/* Check for checkbox and radio buttons */
			switch(el.type.toLowerCase()) {
				case 'checkbox':
				case 'radio':
					value = !!el.checked;
					break;
				default:
					value = el.value;
					break;
			}
			//if( MVC.Object.is_number(value) ) value = parseFloat(value);
			if( key_components.length > 1 ) {
				var last = key_components.length - 1;
				var nested_key = key_components[0].toString();
				if(! data[nested_key] ) data[nested_key] = {};
				var nested_hash = data[nested_key];
				for(var k = 1; k < last; k++){
					nested_key = key_components[k];
					if( ! nested_hash[nested_key] ) nested_hash[nested_key] ={};
					nested_hash = nested_hash[nested_key];
				}
				nested_hash[ key_components[last] ] = value;
			} else {
		        if (key in data) {
		        	if (typeof data[key] == 'string' ) data[key] = [data[key]];
		         	data[key].push(value);
		        }
		        else data[key] = value;
			}
		}
		return data;
	},
    /*
     * Returns the class element for the element selected
     * @return {HTMLElement} the element that shares the controller's id or classname
     */
	class_element : function(){
		var start = this.element;
		var className = this._className();
        var has_class = function(el){
            var parts = el.className.split(" ")
            for(var i =0; i < parts.length; i++){
                if(parts[i] == className) return true;
            }
            return false;
        }
        while(start && !has_class(start) ){
			start = start.parentNode;
			if(start == document) return null;
		}
		return MVC.$E(start);
	},
    /*
     * Returns if the event happened directly on the element in the params.
     * @return {Boolean} true if the event's target is the element, false if otherwise.
     */
	is_event_on_element : function(){ return this.event.target == this.element; },
    _className : function(){
		return this.controller.singularName;
	},
    /**
     * Returns the model instance associated with dom this action acted on.  
     * It finds the class_element, then looks if it has an id that matches
     * <i>modelName</i>_<i>instanceId</i>.  It uses instanceID to look
     * up the instnace in the model's [MVC.Store store].
     * If you change the controller's [MVC.Controller.static.modelName modelName]
     * it will use a different model to look up the data.
     */
    element_instance : function(){
        var ce, matches, model, modelName = this.controller.modelName, id,  matcher = new RegExp("^"+modelName+"_(.*)$");
        if(! (model=MVC.Model.models[modelName])  ) throw "No model for the "+ this.controller.className+ " controller!";
        ce = this.class_element();
        //first check the class
        return Model._find_by_element(ce, modelName, model)
    }
};

if(!MVC._no_conflict && typeof Controller == 'undefined'){
	Controller = MVC.Controller
};
include.set_path('jmvc/plugins/model/jsonp');
include.plugins('model','lang/date','io/jsonp');
include('remote_model');;
include.set_path('jmvc/plugins/model');
include.plugins('lang/class','lang/openajax');
include('simple_store');
include('model');;
include.set_path('jmvc/plugins/model');
// provides very simple storage
// var store = new MVC.Store();
// when its being extended, it should make a new simplestore

/**
 * Provides simple storage for elements.  Replace this store with Gears!
 */
MVC.Store = MVC.Class.extend(
/* @prototype */
{
	/**
	 * 
	 * @param {Object} klass
	 */
    init: function(klass){
		this._data = {};
        this.storing_class = klass;
	},
	/**
	 * 
	 * @param {Object} id
	 */
    find_one: function(id){
        return id ? this._data[id] : null;
	},
	/**
	 * 
	 * @param {Object} obj
	 */
    create: function(obj){
		var id = obj[obj.Class.id];
		this._data[id] = obj;
	},
	/**
	 * 
	 * @param {Object} id
	 */
    destroy: function(id){
		delete this._data[id];
	},
    /**
     * Finds instances using a test function.  If no test function is provided returns all instances.
     * @param {Function} f
     * @return {Array}
     */
    find : function(f){
        var instances = [];
        for(var id in this._data){
            var inst = this._data[id];
            if(!f || f(inst))
                instances.push(inst);
        }
        return instances;
    },
    /**
     * Clears instances
     */
    clear : function(){
        this._data = {};
    },
    /**
     * Returns if there is no instances
     * @return {Boolean}
     */
    is_empty: function() {
		return !this.find().length;
	}
});;
include.set_path('jmvc/plugins/model');
/**
 * Models wrap an application's data layer.  This is done in two ways:
 * <ul>
 *     <li>Requesting data from and interacting with services</li>
 *     <li>Wrap service data with a domain-specific representation</li>
 * </ul>
 * A strong model layer keeps your code organized and maintainable, but it typically
 * is the least developed part of the MVC architecture.  This guide introduces you
 * to the basics of how a model should work.  Note that much of what it demonstrates could be
 * done easier with a different base Model class such as [MVC.Model.Ajax|Model.Ajax].
 * <h2>Services</h2>
 * Your models should be the way you communicate with the server.  Instead of using
 * Ajax/XHR requests directly, perform those requests in a model.
 *
 * For example:
 @code_start
 <pre>
 //instead of:
 new Ajax('/tasks.json', {onComplete: find_tasks_next_week })
 //do this:
 Task.find('all', find_tasks_next_week)

 //instead of
 new Ajax('/tasks/'+id+'/complete.json',{onComplete: task_completed})
 //do this:
 task.complete(task_completed)
 </pre>
 @code_end
 Typically there are two types of services any application connects to:
 <ul>
 <li>Group - operate on many instances.  Ex: getting all tasks for a user.</li>
 <li>Singular - operate on one instance. Ex: completing a task. </li>
 </ul>
 For these types of services, you will want to build them in slightly different ways.
 <h3>Group Services</h3>
 Group services that request data should look like the following:
 @code_start
 <pre>
 Task = MVC.Model.extend('task',
 {
   find : function(params, callback){
     new Ajax('/tasks.json', {onComplete: MVC.Function.bind(function(response){
         //get data into the right format for create_as_existing
         var data =  eval('('+json_string+')');
         //call create_as_existing to create instances
         var instances = this.create_many_as_existing(data);
         //call back with data.
         callback(instances)
     }) })
   }
 },
 {})
 </pre>
 @code_end
 Note this function uses [MVC.Model.static.create_many_as_existing|create_many_as_existing]
 to create new instances.  By using create_many_as_existing, the model will also publish
 [OpenAjax|OpenAjax.hub] messages that can be listed to by controllers.
 <h3>Singular Service</h3>
 Singular services that minipulate data might look like:
 @code_start
 <pre>
 Task = MVC.Model.extend('task',
 {},
 {
   complete: function(callback){
     new Ajax('/tasks/'+this.id+'/complete.json', {onComplete: MVC.Function.bind(function(response){
         this.completed = true;
         callback(this)
         this.publish("completed")
     }) })
   }
 })
 </pre>
 @code_end
 <h2>Wrapping Data</h2>
 Now that you have instances, you can wrap their data in useful ways.  This is done by adding
 functions to the Model's prototype methods.  For example:
 @code_start
 <pre>
 Task = MVC.Model.extend('task',
 {},
 {
   status : function(){
     return this.complete ? "COMPLETE" : "INCOMPLETE"
   }
 })
 </pre>
 @code_end
 * <h3>P</h3>
 * <ul>
 *     <li>Model.find_one(params, callbacks)</li>
 *     <li>Model.find_all(params, callbacks)</li>
 *     <li>Model.create(attributes, callbacks)</li>
 *     <li>Model.update(id, attributes, callbacks)</li>
 *     <li>Model.destroy(id, callbacks)</li>
 * </ul>
 *
 * There three are possible callbacks can be passed to the upper methods:
 * <ul>
 *     <li> onSuccess -- called after onComplete if request was successful
 *     <li> onFailure -- called after onComplete if response has failed
 *     <li> onComplete -- called after a response arrives, typically defined by the model, user should use two previous callbacks
 * </ul>
 *
 * onComplete is 'reserved' for specialized models, like JsonP where status of the response is determined in this callback
 * user though may overwrite this behavior by passing single onComplete callback
 *
 *
 * <h2>Using Stores</h2>
 * Model keeps all instances of a class in a [MVC.Store|Store].  Stores provide an easy way of
 * looking up instances by id.
 *
 * <h2>Using OpenAjax</h2>
 *
 */
MVC.Model = MVC.Class.extend(
    /* @Static*/
    {
        store_type             : MVC.Store,
        /**
         * Creates this model Class instance
         *
         * @constructor
         */
        init                   : function () {
            if (!this.className) return;
            MVC.Model.models[this.className] = this;
            this.store = new this.store_type(this);

            for (var association in this.associations) {
                if (!this.associations.hasOwnProperty(association)) continue;

                switch (association) {
                    case 'has_many':
                        this._has_many.apply(this, this.associations['has_many']);
                        break;
                    case 'has':
                        this._has.apply(this, this.associations['has']);
                        break;
                    //TODO belongs_to
                    default:
                        throw 'Unknown association ' + association;
                }
            }
        },
        /**
         * Creates an instance of this model from a json string
         *
         * @param {String} json
         */
        from_json              : function (json) {
            var attributes = MVC.Object.from_json(json);
            this.create_as_existing(attributes);
        },
        /**
         * Finds objects in this class
         *
         * Inherited models may redefine this method to perform a request
         *
         * @param {Object} id the id of a object
         * @param {Object} params params passed to the
         * @param {Object} cbks callbacks object
         * @return {Model} will return instances of the model if syncronous
         */
        find_one              : function (id, params, cbks) {
            if (!params)  params = {};
            if (typeof params == 'function') {
                cbks = params;
                params = {};
            }
            var callbacks = this._clean_callbacks(cbks);
            var inst = this.store.find_one(id, callbacks);
            if(inst) callbacks.onSuccess(inst);
            return inst;
        },
        /**
         * Lookups for data in the local storage
         *
         * @param {Function|string} f -- filter function or 'all'
         * @return {Array}
         */
        find:function(f){
            if(f == 'all') f = 0;
            return this.store.find(f)
        },
        /**
         * Finds all instances that satisfies filter function or all, if no filter function is provided
         *
         * @param {Function} f a filter function
         * @return {Array}
         */
        find_all               : function (params, cbs) {
            var filter;
            if (typeof params == 'function') filter = params;
            if (params && params.filter && typeof params.filter == 'function') filter = params.filter;
            var callbacks = this._clean_callbacks(cbs);
            var result = this.store.find(filter);
            callbacks.onSuccess(result);
            return result;
        },
        asynchronous           : true,
        /**
         * Creates an instance of this model asynchronously
         *
         * @param {Object} attributes -- new instance's attributes
         * @param {Object} cbks -- callbacks. At least onSuccess must be provided
         */
        create                 : function (attributes, cbks) {
            var callbacks = this._clean_callbacks(cbks);
            callbacks.onSuccess(this.create_as_existing(attributes));
        },
        /**
         * This method does not actually updates values and should not be called directly
         * use update_attributes instance method to really update
         *
         * @param id
         * @param attributes
         * @param {Object} cbks -- callbacks. At least onSuccess must be provided
         */
        update                 : function (id, attributes, cbks) {
            var inst = this.find(id);
            if (!inst) throw "Instance[id=" + id + "] was not found!";
            var callbacks = this._clean_callbacks(cbks);
            this.publish("update", {data: inst});
            callbacks.onSuccess(inst);
        },
        /**
         *
         */
        destroy:function(id, cbks){
            if(this.store.find_one(id)) this.store.destroy(id);
            var callbacks = this._clean_callbacks(cbks);
            this.publish("destroy", {});
            callbacks.onSuccess();
        },
        /**
         * Used to create an existing object from attributes
         *
         * Publishes 'create.as_existing' event
         *
         * @param {Object} attributes
         * @return {Model} an instance of this model
         */
        create_as_existing     : function (attributes) {
            if (!attributes) return null;
            if (attributes.attributes) attributes = attributes.attributes();
            var inst = new this(attributes);
            inst.is_new_record = this.new_record_func;

            this.publish("create.as_existing", {data: inst});

            //if(MVC.Controller) MVC.Controller.publish(this.className + ":create_as_existing", );
            return inst;
        },
        /**
         * Creates instances of this model from an array of attributes
         *
         * @param {Array} instances
         * @return {Array} an array of instances of this model
         */
        create_many_as_existing: function (instances) {
            var res = [];
            if (!instances) return res;
            for (var i = 0; i < instances.length; i++)
                res.push(this.create_as_existing(instances[i]));
            return res;
        },
        /**
         * The name of the id field.  Defaults to 'id'
         */
        id                     : 'id', //if null, maybe treat as an array?
        new_record_func        : function () {
            return false;
        },
        /**
         * An array of functions which will be applied to the instance of this model when validate method is invoked
         *
         * Each function returns an error (simply text) or nothing
         *
         */
        validations            : [],
        _validator             : function (inst) {
            return MVC.Function.bind(
                function () {
                for (var i = 0, size = this.validations.length, f; i < size; ++i) {
                    f = this.validations[i];
                    var error = f(inst);
                    if (error && typeof error == 'string') inst.errors.push(error);
                }
            },this);
        },
        _has_many               : function () {
            for (var i = 0; i < arguments.length; i++) {
                this._associations.push(MVC.String.pluralize(arguments[i]));
            }
        },
        _has                    : function () {
            for (var i = 0; i < arguments.length; i++) {
                this._associations.push(arguments[i]);
            }
        },
        _belongs_to             : function () {
            for (var i = 0; i < arguments.length; i++) {
                this._associations.push(arguments[i]);
            }
        },
        associations           : {
            has_many: [],
            has     : []
        },
        _associations          : [],
        /**
         * Takes an element ID like 'todo_5' and returns '5'
         * @param {Object} element_id
         * @return {String}
         */
        element_id_to_id       : function (element_id) {
            var re = new RegExp(this.className + '_', "i");
            return element_id.replace(re, '');
        },
        /**
         * Returns an instance if one can be found in the store.
         * @param {Object} el
         */
        find_by_element        : function (el) {
            return this._find_by_element(MVC.$E(el), this.className, this);
        },
        _find_by_element       : function (ce, modelName, model) {
            var matches, id, matcher = new RegExp("^" + modelName + "_(.*)$");
            if (ce && ce.id && (matches = ce.id.match(matcher) ) && matches.length > 1) {
                id = matches[1]
            } else {
                id = ce.has_class(matcher)[1]
            }
            return model.store.find_one(id);
        },
        /**
         * Adds an attribute to the list of attributes for this class.
         * @param {String} property
         * @param {String} type
         */
        add_attribute          : function (property, type) {
            //TODO set association
            if (!this.attributes[property])
                this.attributes[property] = type;
            if (!this.default_attributes[property])
                this.default_attributes[property] = null;
        },
        attributes             : {},
        default_attributes     : {},
        /**
         * Used for converting callbacks to to seperate failure and succcess
         * @param {Object} callbacks
         */
        //TODO should we really always return onComplete and onFailure?
        _clean_callbacks       : function (callbacks) {
            if (!callbacks) {
                callbacks = function () {
                };
            }
            if (typeof callbacks == 'function')
                return {onSuccess: callbacks, onFailure: callbacks};
            if (!callbacks.onSuccess && !callbacks.onComplete) throw "You must supply a positive callback!";
            if (!callbacks.onSuccess) callbacks.onSuccess = callbacks.onComplete;
            if (!callbacks.onFailure && callbacks.onComplete) callbacks.onFailure = callbacks.onComplete;
            return callbacks;
        },
        models                 : {},
        /**
         * Creates a callback function that will call back the function on the static class.
         * If other arguments are passed, they will be added before the parameters used to call the callback.
         * @param {String} fname
         * @return {Function} a callback function useful for Ajax calls
         */
        callback               : function (fname) {
            var f = typeof fname == 'string' ? this[fname] : fname;
            var args = MVC.Array.from(arguments);
            args.shift();
            args.unshift(f, this);
            return MVC.Function.bind.apply(null, args);
        },
        /**
         * Publishes to open ajax hub.  Always adds the className.event
         * @param {Object} event
         * @param {Object} data
         */
        publish                : function (event, data) {
            OpenAjax.hub.publish(this.className + "." + event, data);
        },
        /**
         * Namespaces are used to publish messages to a specific namespace.
         * @code_start
         * <pre>
         * Org.Task = MVC.Model.extend('task',{
     *   namespace: "org"
     * },
         * {
     *   update: function(){
     *     this.publish("update") // publishes 'this' to 'org.task.update'
     *   }
     * })
         * </pre>
         * @code_end
         */
        namespace              : null
    },
    /* @Prototype*/
    {
        /**
         * Creates, but does not save a new instance of this class
         * @param {Object} attributes -> a hash of attributes
         */
        init             : function (attributes) {
            //this._properties = [];
            this.errors = [];
            if (attributes.errors) {
                this.add_errors(attributes.errors);
                delete attributes.errors;
            }

            this.set_attributes(this.Class.default_attributes || {});
            this.set_attributes(attributes);
        },
        /**
         * Sets a hash of attributes for this instance
         * @param {Object} attributes
         */
        set_attributes   : function (attributes) {
            for (var key in attributes) {
                if (attributes.hasOwnProperty(key))
                    this._setAttribute(key, attributes[key]);
            }
            return attributes;
        },
        /**
         * Sets the attributes on this instance and calls save.
         * @param {Object} attributes
         * @param {Object} callback
         */
        update_attributes: function (attributes, callback) {
            this.set_attributes(attributes);
            return this.save(callback);
        },
        /**
         * Determines if this instance has no errors
         *
         * @returns {boolean}
         */
        valid            : function () {
            return this.errors.length == 0;
        },
        /**
         * Validates this instance
         */
        validate         : function () {
            this.Class._validator(this)();
        },
        _setAttribute    : function (attribute, value) {
            if (MVC.Array.include(this.Class._associations, this.Class.attributes[attribute]))
                this._setAssociation(attribute, value);
            else
                this._setProperty(attribute, value);
        },
        /**
         * Checks if there is a set_<i>property</i> value.  If it returns true, lets it handle; otherwise
         * saves it.
         * @param {Object} property
         * @param {Object} value
         */
        _setProperty     : function (property, value) {
            if (this["set_" + property] && !this["set_" + property](value)) {
                return;
            }
            //add to cache, this should probably check that the id isn't changing.  If it does, should update the cache
            var old = this[property];


            this[property] = MVC.Array.include(['created_at', 'updated_at'], property) ? MVC.Date.parse(value) : value;
            if (property == this.Class.id && this[property]) {
                this.is_new_record = this.Class.new_record_func;
                if (this.Class.store) {
                    if (!old) {
                        this.Class.store.create(this);
                    } else if (old != this[property]) {
                        this.Class.store.destroy(old);
                        this.Class.store.create(this);
                    }
                }

            }
            //if (!(MVC.Array.include(this._properties,property))) this._properties.push(property);
            else
                this.Class.add_attribute(property, MVC.Object.guess_type(value));
        },
        _setAssociation  : function (attribute, values) {
            var association = this.Class.attributes[attribute];

            var hasMany = !MVC.String.is_singular(association);

            var associated_class = (hasMany) ? window[MVC.String.singularize(association)] : window[association];
            if (!associated_class)
                this[attribute] = values;
            else if (hasMany)
                this[attribute] = associated_class.create_many_as_existing(values);
            else
                this[attribute] = associated_class.create_as_existing(values);
        },
        /**
         * Returns this model's attributes
         *
         * Associated models include their attributes as nested objects into the result
         *
         * @return {Object}
         */
        attributes       : function () {
            var attributes = {};
            var cas = this.Class.attributes;
            for (var attr in cas) {
                if (cas.hasOwnProperty(attr)) {
                    if (MVC.Array.include(this.Class._associations, this.Class.attributes[attr]))
                        attributes[attr] = this[attr].attributes();
                    else
                        attributes[attr] = this[attr];
                }
            }
            return attributes;
        },
        /**
         * Returns if the instance is a new object
         */
        is_new_record    : function () {
            return true;
        },
        /**
         * Saves the instance
         * @param {Function} callbacks onComplete function or object of callbacks
         */
        save             : function (callbacks) {
            var result;
            this.errors = [];
            this.validate();
            if (!this.valid()) return false;
            result = this.is_new_record() ?
                this.Class.create(this.attributes(), callbacks) :
                this.Class.update(this[this.Class.id], this.attributes(), callbacks);

            this.is_new_record = this.Class.new_record_func;
            return true;
        },
        /**
         * Destroys this instance
         *
         * @param {Function} callback or object of callbacks
         */
        destroy          : function (callback) {
            this.Class.store.destroy(this[this.Class.id]);
            this.Class.destroy(this[this.Class.id], callback);
        },
        /**
         * Add  errors to this instance
         *
         * @param {Array} errors
         */
        add_errors       : function (errors) {
            //TODO accept var args
            if (errors) this.errors = this.errors.concat(errors);
        },
        _resetAttributes : function (attributes) {
            this._clear();
            /*for (var attr in attributes){
             if(attributes.hasOwnProperty(attr)){
             this._setAttribute(attr, attributes[attr]);
             }
             }*/
        },
        _clear           : function () {
            var cas = this.Class.default_attributes;
            for (var attr in cas) {
                if (cas.hasOwnProperty(attr)) this[attr] = null;
            }
        },
        /**
         * Returns the suggested element id for this instance
         */
        element_id       : function () {
            return this.Class.className + '_' + this[this.Class.id];
        },
        /**
         * Returns an html element found by using element_id for this instance
         *
         * @returns {*}
         */
        element          : function () {
            return MVC.$E(this.element_id());
            ;
        },
        /**
         * Returns html elements with this element_id class
         *
         * @returns {*}
         */
        elements         : function () {
            return MVC.Query("." + this.element_id());
        },
        /**
         * Publishes to open ajax hub
         * @param {String} event
         * @param {optional:Object} data if missing, uses the instance in {data: this}
         */
        publish          : function (event, data) {
            this.Class.publish(event, data || {data: this});
        },
        /**
         * Creates a callback function that will call back the function on the instance.
         * If other arguments are passed, they will be added before the parameters used to call the callback.
         * @param {String} fname
         * @return {Function} a callback function useful for Ajax calls
         */
        callback         : function (fname) {
            var f = typeof fname == 'string' ? this[fname] : fname;
            var args = MVC.Array.from(arguments);
            args.shift();
            args.unshift(f, this);
            return MVC.Function.bind.apply(null, args);
        }
    });


MVC.Object.guess_type = function (object) {
    if (typeof object != 'string') {
        if (object == null) return typeof object;
        if (object.constructor == Date) return 'date';
        if (object.constructor == Array) return 'array';
        return typeof object;
    }
    //check if true or false
    if (object == 'true' || object == 'false') return 'boolean';
    if (!isNaN(object)) return 'number'
    return typeof object;
}

if (!MVC._no_conflict && typeof Model == 'undefined') {
    Model = MVC.Model;
};
include.set_path('jmvc/plugins/lang/date');

(function(){
	var parse = Date.parse;
	/**
	 * @class MVC.Native.Date
	 * @alias MVC.Date
	 * Adds Date helpers for dates.
	 * <h3>Install</h3>
	 * @code_start
	 * include.plugins('lang/date')
	 * @code_end
	 */
    MVC.Native.extend('Date', {
        /**
         * @plugin lang/date
         * @param {Object} date
         * @param {Object} number_of_days
         */
		add_days: function(date, number_of_days) {
			date.setDate(date.getDate() + number_of_days);
			return date;
		},
        /**
         * @plugin lang/date
         * @param {Object} date
         * @param {Object} number_of_weeks
         */
		add_weeks: function(date, number_of_weeks) {
			return MVC.Date.add_days(date, number_of_weeks * 7);
		},
        /**
         * @plugin lang/date
         * @param {Object} date
         */
		day_name: function(date) {
			return MVC.Date.day_names[date.getDay()];
		},
        /**
         * @plugin lang/date
         * @param {Object} date
         */
		first_day_of_week: function(date) {
			var first_day = new Date(date);
			first_day.setDate(date.getDate() - date.getDay());
			return first_day;
		},
        /**
         * @plugin lang/date
         * @param {Object} date
         */
		month_name: function(date) {
			return MVC.Date.month_names[date.getMonth()];
		},
        /**
         * @plugin lang/date
         * @param {Object} date
         */
		number_of_days_in_month : function(date) {
		    var year = date.getFullYear(),month = date.getMonth(),m = [31,28,31,30,31,30,31,31,30,31,30,31];
		    if (month != 1) return m[month];
		    if (year%4 != 0 || (year%100 == 0 && year%400 != 0)) return m[1];
		    return m[1] + 1;
		},
        /**
         * @plugin lang/date
         */
		day_names: ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"],
        /**
         */
		month_names: ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"],
		/**
		 * @param {Object} data
		 */
        parse: function(data) {
			if(typeof data != "string") return null;
			var f1 = /\d{4}-\d{1,2}-\d{1,2}/, f2 = /\d{4}\/\d{1,2}\/\d{1,2}/, f3 = /(\d{4})[-\/](\d{1,2})[-\/](\d{1,2})\s(\d{1,2}):(\d{1,2}):(\d{1,2})/;
			var match_data;
			if((match_data =  data.match(f3) ) ) {
				return new Date( Date.UTC(parseInt(match_data[1], 10), (parseInt(match_data[2], 10)-1), parseInt(match_data[3], 10),
					parseInt(match_data[4], 10), parseInt(match_data[5], 10), parseInt(match_data[6], 10)) );
			}
			if(data.match(f1)) {
				var dateArr = data.match(f1)[0].split('-');
				return new Date( Date.UTC(parseInt(dateArr[0], 10), (parseInt(dateArr[1], 10)-1), parseInt(dateArr[2], 10)) );
			}
			if(data.match(f2)) {
				var dateArr = data.match(f2)[0].split('/');
				return new Date( Date.UTC(parseInt(dateArr[0], 10), (parseInt(dateArr[1], 10)-1), parseInt(dateArr[2], 10)) );
			}
			return parse(data);
		}
	});
})();
MVC.Native.Date = MVC.Date;
include.set_path('jmvc/plugins/io/jsonp');
include.plugins('lang');
include('jsonp');;
include.set_path('jmvc/plugins/io/jsonp');
/**
 * Provides JSONP functionality similar to [MVC.IO.Ajax MVC.Ajax]
 * Available options are:
 * <ul>
 *     <li>remove_script [boolean] -- indicates whether script tag will be removed after executing callbacks
 *     <li>error_timeout [int](ms) -- how long the script will wait for a response
 *     <li>callback_name [String] -- callback parameter name
 *     <li>session [function, string] -- session id
 *     <li>method [string] -- http method, must be supported by the server side
 *     <li>onSuccess [function(response)] -- positive callback (if server responses in time)
 *     <li>onFailure [function(response)] -- failure callback (in case timeout)
 * </ul>
 *
 * @constructor MVC.IO.JsonP
 * @alias MVC.JsonP
 */
MVC.JsonP = function(url, options){
    this.url = url;
    this.options = options || {};
    if(!this.options.callback_name) this.options.callback_name = "cbk";
    this.remove_script = this.options.remove_script == false ? false : true;
    this.options.parameters = this.options.parameters || {};
    this.error_timeout = this.options.error_timeout || 7000;
    this.send();
    
}

MVC.JsonP.prototype = {
    send : function(){
        var n = 'c'+MVC.get_random(5);

        if(this.options.session){
            var session = typeof this.options.session == 'function' ? this.options.session() : this.options.session;
            this.url += (MVC.String.include(this.url,';') ? '&' : ';') + MVC.Object.to_query_string(session);
        }

        var params  = typeof this.options.parameters == 'function' ? this.options.parameters() : this.options.parameters; //will eval everytime
        

        this.url += (MVC.String.include(this.url,'?') ? '&' : '?') + MVC.Object.to_query_string(params);
        this.add_method();

        var callback_name = this.callback_and_random(n);
        
        var error_callback = this.options.onFailure;
        var error_timer = this.check_error(this.url, error_callback);
        
        MVC.JsonP._cbs[callback_name] = MVC.Function.bind(function(callback_params){
            clearTimeout(error_timer);
			this._remove_scripts();
            delete MVC.JsonP._cbs[callback_name];

            //convert to a transport
            var transport = {};
            if(callback_params == null){
            	transport.responseText = "";
            }else if(typeof callback_params == 'string'){
            	transport.responseText = callback_params
                //TODO assume that this is an internal error of the server?!
            }else{
            	transport = callback_params;
            	transport.responseText = callback_params.toString();
            }

            if (this.options.onSuccess)
               this.options.onSuccess(transport);
		},this);
        include({path: this.url});
    },
    add_method : function(){
        if(this.options.method && this.options.method != 'get') this.url += "&_method="+this.options.method;
    },
    callback_and_random : function(n){
        
        
        //if(this.options.callback){
        //    this.url += "&callback=" +this.options.callback+"&"+n
        //    return this.options.callback;
        //}
        this.options.callback = "MVC.JsonP._cbs."+n;
        this.url += "&" + this.options.callback_name + "=" +this.options.callback;
        return n;
    },
    check_error : function(url, error_callback){
        return setTimeout(function(){
            console.error("TIMEOUT from:"+url);
            if(error_callback)
                error_callback({
                    responseText:"No response from the server!"
                });
        }, this.error_timeout)
    },
    /**
     * Removes script tag associated with this JSONP
     */
    remove_scripts : function(){
        this.remove_script = true;
        setTimeout(MVC.Function.bind(this._remove_scripts,this), 2000 )
    },
    _remove_scripts : function(){
        if(!this.remove_script) return;
        var scripts = document.getElementsByTagName('script');
        var search = new RegExp(this.url);
        for(var s = 0; s < scripts.length; s++){
            var script = scripts[s];
            if(MVC.String.include( script.src.toLowerCase() ,this.url.toLowerCase())) script.parentNode.removeChild(script);
        }
    }
}
MVC.JsonP._cbs = {};
MVC.IO.JsonP = MVC.JsonP;;
include.set_path('jmvc/plugins/model/jsonp');
/**
 * Model for connecting to resources with JSONP
 *
 * Since JSONP does not really have a way to distinguish between timeout (server is not available) and any exception on the server
 * side (Resource not found, unhandled ServletException etc) this model uses onComplete callback to decide between onSuccess and onFailure.
 * Therefore user must not define onComplete callback in the request
 *
 * It is expected that if server answers in a normal way then instances of this model will be created and user
 * may verify that there is no errors by checking corresponding field of this model. onFailure may be also
 * called if response contains an array of errors.
 *
 */
MVC.Model.JsonP = MVC.Model.extend(
    {
        _methods:{
            find_one:'get',
            find_all:'get',
            create:'post',
            update:'put',
            destroy:'delete'
        },
        /**
         * How long to wait before claiming "no response from server"
         */
        error_timeout: 3000,
        /**
         * Constructor for this model's Class
         *
         * @constructor
         */
        init: function () {
            if (!this.className) return;
            if (!this.domain) throw('a domain must be provided for remote model');
            if (!this.controller_name)
                this.controller_name = this.className;
            this.plural_controller_name = MVC.String.pluralize(this.controller_name);
            this._super();
        },
        /**
         * Looks for an instance of this model in {MVC.Store},
         * if not found -- requests server
         *
         * @param {*} id -- id to look up
         * @param {Object} params
         * @param {Object|Function} cbs -- callbacks
         * @return {MVC.Model}
         */
        find_one: function (id, params, cbs) {
            if(typeof params == 'function'){
                cbs = params;
                params = {};
            }
            var inst = this._super(id, params, cbs);

            if(inst) return inst;

            params[this.id] = id;
            this._send_request('find_one',params, cbs);
        },
        _default_onFailure:function(response){
            console.error(response.errors);
        },
        /**
         * Wraps user defined onFailure
         *
         * @param error_callback
         * @private
         */
        _error_callback:function(error_callback){
            return function(response){
                error_callback(MVC.Object.extend(response,{errors:response.responseText}));
            }
        },
        /**
         * Request all instances of this model from the server.
         *
         * Instances all looked up at this.find_url or this.domain/this.plural_controller_name
         *
         * One can provide the following callbacks:
         * <ul>
         *     <li> onSuccess -- invoked in case of a successful response, i.e. received in time and without errors
         *     <li> onFailure -- invoked if response has errors. May not be provided -- in this case errors will be simply ignored
         * </ul>
         *
         * @param {Object} params
         * @param {Object} cbs
         */
        find_all: function (params, cbs) {
            params = params || {};

            this._send_request('find_all',params,cbs);
        },
        /**
         * Same as this._onComplete but always passes an array of this model instances
         *
         * @param {Object} clean_callbacks -- user callbacks passed through _clean_callbacks
         * @returns {Function}
         * @private
         */
        _find_all_onComplete:function(clean_callbacks){
            var me = this;
            return function (response) {
                var insts = me.create_many_as_existing(response);
                if (response.errors)
                    clean_callbacks.onFailure(insts);
                else
                    clean_callbacks.onSuccess(insts);
            };
        },
        _onComplete:function(clean_callbacks, action){
            var me = this;
            return function (response) {
                var inst = new me(response);
                me.publish(action,{data:inst});
                if (inst.valid())
                    clean_callbacks.onSuccess(inst);
                else
                    if(clean_callbacks.onFailure) clean_callbacks.onFailure(inst);

            };
        },
        /**
         * Creates a new instance of this model on the server.
         *
         * The instance is created at this.create_url or this.domain/this.plural_controller_name
         *
         * One can provide the following callbacks:
         * <ul>
         *     <li> onSuccess -- invoked in case of a successful response, i.e. received in time and without errors
         *     <li> onComplete -- invoked in both cases: success or failure
         *     <li> onFailure -- invoked if response has errors. May not be provided -- in this case errors will be simply ignored
         * </ul>
         *
         * @param {Object} params
         * @param {Object} cbs
         */
        create: function (params, cbs) {
            this._send_request('create', params, cbs);
        },
        /**
         * Updates an instance of this model on the server
         *
         * @param id
         * @param {Object} attributes
         * @param {Object} cbs
         */
        update: function (id, attributes, cbs) {
            var params = {};

            params[this.id] = id;
            MVC.Object.extend(params, attributes);

            this._send_request('update',params,cbs);
        },
        /**
         * Destroys an instance of this model on the server.
         *
         * Instance is destroyed at this.destroy_url or this.domain/this.plural_controller_name
         *
         * @param id
         * @param {Object} cbs
         */
        destroy: function (id, cbs) {
            var params = {};

            params[this.id] = id;

            this._send_request('destroy',params,cbs);
        },
        _send_request:function(action, params, cbs){
            this._add_standard_params(params, action);

            var callbacks = this._clean_callbacks(cbs);
            if(!callbacks.onFailure) callbacks.onFailure = this._default_onFailure;
            var callback;
            if(action == 'find_all')
                callback = this._find_all_onComplete(callbacks);
            else if(action == 'create')
                callback = this._single_create_callback(callbacks);
            else
                callback = this._onComplete(callbacks,action);
            var error_callback = this._error_callback(callback);

            var url = this[action + '_url'] ? this[action + '_url'] : this.domain + '/' + this.plural_controller_name;
            url += '?';

            var tll = this.top_level_length(params, url);
            var result = this._separate(params[this.controller_name], tll, this.controller_name);
            var postpone_params = result.postpone, send_params = result.send;

            if (result.send_in_parts) {
                //TODO test
                params[this.controller_name] = send_params;
                params['_mutlirequest'] = 'true';

                new MVC.JsonP(url, {
                    error_timeout: this.error_timeout,
                    callback_name: this.callback_name,
                    parameters: params,
                    onSuccess: MVC.Function.bind(this._parts_create_callback(params, callback, postpone_params), this),
                    onFailure: error_callback,
                    method: this._methods[action]
                });
            } else {
                params['_mutlirequest'] = null;

                new MVC.JsonP(url, {
                    error_timeout: this.error_timeout,
                    callback_name: this.callback_name,
                    parameters: params,
                    onSuccess: callback,
                    onFailure: error_callback,
                    method: this._methods[action]
                });
            }
        },
        _parts_create_callback: function (params, callback, postpone_params) {
            var me = this;
            return function (callback_params) {
                if (!callback_params.id) throw 'Your server must callback with the id of the object.  It is used for the next request';
                params[me.controller_name] = postpone_params;
                params.id = callback_params.id;
                me.create(params, callback);
            };
        },
        _single_create_callback: function (clean_callbacks) {
            var me = this;
            return function (response) {
                if (response.errors) {
                    var inst = new me(response[me.className] ? response[me.className] : {});
                    inst.add_errors(response.errors);
                    clean_callbacks.onFailure(inst);
                } else {
                    clean_callbacks.onSuccess(me.create_as_existing(response));
                }
            };
        },
        /**
         * //TODO redefine server error protocol
         *
         * Server always returns single object in case of error.
         *
         * This method works around this limitation.
         *
         * @param instances an array of raw objects from the server
         * @return {Array} an array of newly created objects
         */
        create_many_as_existing: function (instances) {
            if (!instances) return [];
            if (instances.errors) return [this.create_as_existing(instances)];

            return this._super(instances);
        },
        _add_standard_params: function (params, action) {
            if (!params.referer) params.referer = window.location.href;
            if (!params.action) params.action = action;
        },
        callback_name: 'cbk',
        domain: null,
        top_level_length: function (params, url) {
            var p = MVC.Object.extend({}, params);
            delete p[this.controller_name];
            return url.length + MVC.Object.to_query_string(p).length;

        },
        _separate: function (object, top_level_length, name) {
            var remainder = 2000 - 9 - top_level_length;
            var send = {};
            var postpone = {};
            var send_in_parts = false;
            for (var attr in object) {
                if (!object.hasOwnProperty(attr)) continue;
                var value = object[attr], value_length;
                var attr_length = encodeURIComponent(name + '[' + attr + ']').length;

                if (typeof value == 'string') {
                    value_length = encodeURIComponent(value).length;
                } else {
                    value_length = value.toString().length;
                }

                if (remainder - attr_length <= 30) {
                    postpone[attr] = value;
                    send_in_parts = true;
                    continue;
                }
                ;
                remainder = remainder - attr_length - 2; //2 is for = and &
                if (remainder > value_length) {
                    send[attr] = value;
                    remainder -= value_length;
                } else if (typeof value == 'string') {
                    var guess = remainder;
                    while (encodeURIComponent(value.substr(0, guess)).length > remainder) {
                        guess = parseInt(guess * 0.75) - 1;
                    }
                    send[attr] = value.substr(0, guess);
                    postpone[attr] = value.substr(guess);
                    send_in_parts = true;
                    remainder = 0;
                } else {
                    postpone[attr] = value;
                }
            }
            return {send: send, postpone: postpone, send_in_parts: send_in_parts};
        },
        random: parseInt(Math.random() * 1000000)
    },
//prototype functions
    {});
;
include.next_function();
include.set_path('models');

/**
 * Provides similar functionality as org.tango.client.DeviceProxy from TangORB
 *
 * @type {mtango.DeviceProxy}
 */
mtango.DeviceProxy = MVC.Model.extend('DeviceProxy',
/* @Static */
{
    /**
     * For internal use only
     */
    _stateEnum : Object.freeze({
        UNDEFINED   : 'UNDEFINED',
        INITIAL     : 'INITIAL',
        CONTINUATION: 'CONTINUATION'
    }),
    kDeviceProxyDelay:200,
    kDeviceProxyRequestValidTime:200,
    api:'mtango',
    id:'device',
    error_timeout: 5000, // 5s
    attributes:{
        __requests:'Object',
        __subscriptions:'Object',
        __pendingRequests:'Object'//url -> [awaiting requests]
    },
    default_attributes:{//this will be shared across all instances
        __requests:{},
        __subscriptions:{},
        __pendingRequests:{}
    },
    _subscriptionId : 1,
    //create URL
    //init method
    /**
     *
     * @param url
     * @param cmd
     * @param params
     * @param cbs
     * @private
     */
    _sendRequest:function(url, params, cbs){
        console.debug("Sending request: " + url);

        var onSuccess = function(response){
                response.src = url;
                response = new mtango.Response(response);
                if(response.valid()) {
                    console.debug('Request has succeed: ' + url);
                    cbs.onSuccess(response);
                }
                else {
                    console.debug('Request has failed:' + url);
                    response.errors.forEach(function(error){
                        console.error(error.severity + ": " + error.reason + " - " + error.description)
                    });
                    cbs.onFailure(response);
                }
            };
        var onFailure = function(response){
            onSuccess({
                errors:[{
                    reason: "Timeout",
                    severity: "ERR",
                    description: response.responseText,
                    origin: "DeviceProxy.js#_sendRequest"
                }],
                quality:mtango.AttrQualityEnum.INVALID,
                timestamp:Date.now()
            })
        };

        var method = params.method || 'get';
        var timeout = params.timeout || this.error_timeout;


        new MVC.JsonP(url, {
            error_timeout : timeout,
            parameters:params,
            onSuccess :onSuccess,
            onFailure :onFailure,
            method    :method
        });
    },
    /**
     * Resends after getting response
     *
     * @param {string} url
     * @param {Object} params
     * @param {Function} onComplete
     * @private
     */
    _sendRequestContinuously:function(url, params, onComplete){
        var callbacks =  MVC.Model._clean_callbacks(
            MVC.Function.bind(
            function(response) {
                if (onComplete(response))
                    setTimeout(
                        MVC.Function.bind(
                            function(){
                                params.state = this._stateEnum.CONTINUATION;
                                this._sendRequest(url, params, callbacks);
                            }
                        ,this),this.kDeviceProxyDelay);
            },this)
        );
        params.state = this._stateEnum.INITIAL;
        this._sendRequest(url,params, callbacks);
    }

},
/* @Prototype */
{
    device:null,
    url:null,
    full_url:null,
    wrapped_by:null,
    set_url:function(v){
        this.url = v;
    },
    set_wrapped_by:function(v){
        this.wrapped_by = v;
    },
    /**
     *
     * @param attributes {url,device}
     * @constructor
     */
    init: function(attributes){
        this._super(attributes);
        this.full_url = this.url + "/" + this.Class.api + "/device/" + this.device + "/";
    },
    /**
     * Checks request status and acts accordingly
     *
     * Possible callbacks:
     * <ul>
     *      <li>params.onSuccess -- defines positive callback
     *      <li>params.onFailure -- defines negative callback
     * </ul>
     *
     * @param {string} url
     * @param {Action} cmd
     * @param {Object} params
     * @param {Object} cbs
     * @private
     */
    _sendRequest:function(url,cmd,params, cbs){
        var callbacks = MVC.Model._clean_callbacks(cbs);
        var onSuccess = callbacks.onSuccess;
        var onFailure  = callbacks.onFailure;

        var requestObj = this.__requests[url];
        if(!requestObj){
            //create new request object
            this.__requests[url] = new mtango.Request({status:RequestStateEnum.PENDING,cmd:cmd,result:null});

            //redefine callbacks
            callbacks.onSuccess = MVC.Function.bind(function(response){
                //set requestObj status
                this.__requests[url].result = response;
                this.__requests[url].status = RequestStateEnum.FINISHED;
                //remove cache after a while
                setTimeout(MVC.Function.bind(function(){
                    delete this.__requests[url];
                },this),this.Class.kDeviceProxyRequestValidTime);
                //invoke requests cbks
                if(response.valid()) {
                    OpenAjax.hub.publish(this.device + '/' + cmd.target + '.' + cmd.type + '.succeed', response);
                    onSuccess(response);
                } else {
                    OpenAjax.hub.publish(this.device + '/' + cmd.target + '.' + cmd.type + '.failed', response);
                    onFailure(response);
                }
                //invoke awaiting requests
                if(this.__pendingRequests[url] && this.__pendingRequests[url].length > 0) {
                    do{
                        this.__pendingRequests[url].pop()(response);
                    } while(this.__pendingRequests[url].length > 0);
                }
            },this);
            callbacks.onFailure = MVC.Function.bind(function(response){
                //set requestObj status
                this.__requests[url].result = response;
                this.__requests[url].status = RequestStateEnum.ERROR;
                //remove cache after a while
                setTimeout(MVC.Function.bind(function(){
                    delete this.__requests[url];
                },this),this.Class.kDeviceProxyRequestValidTime);
                //requestObj has failed
                OpenAjax.hub.publish(this.device+'/'+cmd.target+'.'+cmd.type+'.failed',response);
                onFailure(response);
                //invoke awaiting requests
                if(this.__pendingRequests[url] && this.__pendingRequests[url].length > 0) {
                    do{
                        this.__pendingRequests[url].pop()(response);
                    } while(this.__pendingRequests[url].length > 0);
                }
            },this);

            //send
            this.Class._sendRequest(url, params, callbacks);
        } else if(requestObj.isPending()){
            if(!this.__pendingRequests[url]) this.__pendingRequests[url] = [];
            this.__pendingRequests[url].push(
                function(response){
                    if(response.errors)
                        onFailure(response);
                    else
                        onSuccess(response)
                });
        } else if(requestObj.isFinished()){
            onSuccess(requestObj.result);
        } else if(requestObj.isError()){
            onFailure(requestObj.result);
        } else {
            console.error("Unreachable statement!");
            throw "Unreachable statement!"
        }
    },
    /**
     * Reads an attribute from the remote tango device
     *
     * @param {string} attributeName
     * @param {Object} params
     * @param {Object|Function} cbks -- callback(s)
     * @asynchronous
     */
    readAttribute:function(attributeName, params, cbks){
        var cmd = Action.createReadAction(attributeName);

        this._sendRequest(this.full_url + cmd.target, cmd, params, cbks);
    },
    /**
     * Reads an attribute info from the remote server
     *
     * @param {string} attributeName
     * @param {Object} params
     * @param {Object|Function} cbks -- callback(s)
     * @asynchronous
     */
    readAttributeInfo:function(attributeName,params, cbks){
        var cmd = mtango.Action.createReadAction(attributeName);

        this._sendRequest(this.full_url + cmd.target + "/info",cmd,params,cbks);
    },
    /**
     * Writes an attribute to the remote tango device
     *
     * @param {string} attributeName
     * @param {Object} params
     * @param {Object|Function} cbks -- callback(s)
     * @asynchronous
     */
    writeAttribute:function(attributeName, params, cbks){
        var cmd = mtango.Action.createWriteAction(attributeName,params.argin);
        var arg = encodeURI(cmd.argin).replace(/\./g,"%2E");
        this._sendRequest(this.full_url + cmd.target + "=" + arg, cmd, MVC.Object.extend(params, {method:'put'}), cbks);
    },
    /**
     * Executes command on a remote tango device
     *
     * @param {string} commandName
     * @param {Object} params
     * @param {Object|Function} cbks -- callback(s)
     * @asynchronous
     */
    executeCommand:function(commandName, params, cbks){
        var cmd = mtango.Action.createExecAction(commandName,params.argin || null);
        this._sendRequest(this.full_url + cmd.target + ((cmd.argin) ? "=" + encodeURI(cmd.argin).replace(/\./g,"%2E"): ""), cmd, params, cbks);
    },
    /**
     * Reads a command info from the remote tango device
     *
     * @param {string} commandName
     * @param {Object} params
     * @param {Object|Function} cbks -- callback(s)
     * @asynchronous
     */
    readCommandInfo:function(commandName, params, cbks){
        var cmd = mtango.Action.createReadAction(commandName);

        this._sendRequest(this.full_url + cmd.target + "/info",cmd,params,cbks);
    },
    /**
     * Subscribes this device proxy to a specified event
     *
     * @param attr
     * @param {mtango.EventTypeEnum} evt
     * @param {Function} handler
     * @param {int} timeout
     * @return {int} subscription id
     * @asynchronous
     */
    subscribe:function(attr, evt, handler, timeout){
        var subscription = this.device + "/" + attr + "." + evt;

        var id = this.Class._subscriptionId++;
        if(this.__subscriptions[subscription]){
            this.__subscriptions[subscription][id] = handler;
            this.__subscriptions[subscription].count++;
        } else {
            this.__subscriptions[subscription] = new Subscription(id,handler);
            this.Class._sendRequestContinuously(this.full_url + attr + "." + evt , {
                    timeout: timeout
                },
                MVC.Function.bind(function(response){
                    if(this.__subscriptions[subscription]){
                        for(var _id in this.__subscriptions[subscription]){
                            if(!this.__subscriptions[subscription].hasOwnProperty(_id) || _id == 'count') continue;
                            this.__subscriptions[subscription][_id](response);
                        }
                        return true;
                    } else
                        return false;
                },this)
            );
        }
        return id;
    },
    /**
     *
     * @param attr
     * @param {mtango.EventTypeEnum} evt
     * @param {int} id -- subscription id obtained in subscribe
     */
    unsubscribe:function(attr, evt, id){
        var subscription = (this.device + "/" + attr + "." + evt);

        if(typeof this.__subscriptions[subscription] == 'undefined') return;

        delete this.__subscriptions[subscription][id];
        this.__subscriptions[subscription].count--;
        if(this.__subscriptions[subscription].count == 0)
            delete this.__subscriptions[subscription];
    },
    /**
     * Shorthand to read state attribute from the device
     *
     * @param {Object} params
     * @param {Object|Function} cbks
     * @asynchronous
     */
    state:function(params, cbks){
        this.readAttribute('State', params, cbks);
    }
});

Subscription = function(id,handler){
    this.count = 1;
    this[id] = handler;
};

if(typeof DeviceProxy == 'undefined') DeviceProxy = mtango.DeviceProxy;;
include.set_path('models');
 /**
 * TODO merge with request
  *
  * @type {mtango.Action}
 */
mtango.Action = MVC.Model.extend('Action',
/* @Static */
{
    attributes:{
        type:'string',//read,write,execute
        target:'string',//attribute or command name
        argin:'object'
    },
    /**
     *
     * @param {String} target matches to attribute or command name of the remote tango server
     * @return {Action}
     */
    createReadAction:function(target){
        return new Action({
            type:'read',
            target:target,
            argin:null
        });
    },
    /**
     *
     * @param {String} target
     * @param {Object} argin or null
     * @return {Action}
     */
    createWriteAction:function(target,argin){
        return new Action({
            type:'write',
            target:target,
            argin:argin
        });
    },
    /**
     *
     * @param {String} target
     * @param {Object} argin or null
     * @return {Action}
     */
    createExecAction:function(target,argin){
        return new Action({
            type:'execute',
            target:target,
            argin:argin
        });
    }

},
/* @Prototype */
{
    toJson:function(){
        return MVC.Object.to_json(this.attributes());
    }
}
);

 if(typeof Action == 'undefined') Action = mtango.Action;;
include.set_path('models');
/**
*
*/
mtango.RequestStateEnum = Object.freeze({
    PENDING:0,
    FINISHED:1,
    ERROR:2
});

/**
 *
 * @type {mtango.Request}
 */
mtango.Request = MVC.Model.extend('Request',
/* @Static */
{
    attributes:{
        status:'Object',
        cmd:'Action',
        result:'Object'
    }
},
/* @Prototype */
{
    isPending:function(){
        return this.status == mtango.RequestStateEnum.PENDING;
    },
    isFinished:function(){
        return this.status == mtango.RequestStateEnum.FINISHED;
    },
    isError:function(){
        return this.status == mtango.RequestStateEnum.ERROR;
    }
});

if(typeof RequestStateEnum == 'undefined') RequestStateEnum = mtango.RequestStateEnum;
if(typeof Request == 'undefined') Request = mtango.Request;;
include.set_path('models');
mtango.AttrQualityEnum = Object.freeze({
    VALID:'VALID',
    INVALID:'INVALID',
    ALARM:'ALARM',
    CHANGING:'CHANGING',
    WARNING:'WARNING'
});

/**
 *
 * @type {mtango.Response}
 */
mtango.Response = MVC.Model.extend('Response',
    /* @Static@ */
    {
        attributes:{
            src:'URI',
            argout:'*',
            quality:'string',
            timestamp:'long'
        },
        default_attributes:{}
    },
    /* @Prototype@ */
    {}
);

if(typeof AttrQualityEnum == 'undefined') AttrQualityEnum = mtango.AttrQualityEnum;
if(typeof Response == 'undefined') Response = mtango.Response;;
include.set_path('models');
/**
 * Defines for how long DeviceProxy will keep responses
 *
 * @type {number}
 */
mtango.EventTypeEnum = Object.freeze({
    CHANGE:"change",
    PERIODIC:"periodic",
    ARCHIVE:"archive",
    USER:"user"
});
/**
 * Model
 *
 * @type {mtango.DeviceAttribute}
 */
mtango.DeviceAttribute = MVC.Model.extend('DeviceAttribute',
    /* @Static */
    {
        _ID: 1,//global counter
        attributes: {
            name : "string",
            label: "string",
            description: "string",
            isReadOnly: "boolean",
            dataFormat: "string",
            dataType: "string",
            unit    : "string",
            displayUnit: "string",
            maxDimX: "int",
            maxDimY: "int",
            maxValue: "string",
            minValue: "string"
        },
        default_attributes: {},
        /**
         * Creates a new instance of DeviceAttribute and populates it's properties (name, label etc) using network call
         *
         * @param {String} device
         * @param {String} attr_name
         * @param {Object} params callbacks
         * @asynchronous
         */
        create:function(device, attr_name, params) {
            var callbacks = this._clean_callbacks(params);

            var proxy = mtango.DeviceProxy.find(device);
            if (proxy == null) throw "Can not create DeviceAttribute! No proxy was found for device: " + device;
            new MVC.JsonP(proxy.full_url + attr_name + "/info", {
                onComplete: MVC.Function.bind(function (response) {
                    var attributes = {
                        id: this._ID++,
                        proxy: proxy
                    };
                    MVC.Object.extend(attributes, response);
                    var instance = this.create_as_existing(attributes);
                    if (instance.valid())
                        callbacks.onComplete(instance);
                    else
                        callbacks.onFailure(instance);
                }, this)
            });
        }
    },
    /* @Prototype */
    {
        timeout:5000,
        proxy: null,
        response: null,
        _subscriptions: null,
        /**
         * Sets a proxy backend for this attribute
         *
         * @param v
         */
        set_proxy:function(v) {
            this.proxy = v;
        },
        /**
         *
         * @param {mtango.Response} v
         */
        set_response:function(v){
            this.value = v;
        },
        /**
         *
         * @param {int} v
         */
        set_timeout:function(v){
            this.timeout = v;
        },
        /**
         * Returns the suggested element id for this instance
         */
        element_id:function(){
            return this.proxy.device+'/'+this.name+'_'+this.id;
        },
        /**
         *
         * @param attributes
         * @constructor
         */
        init:function(attributes){
            if(!attributes.id) attributes.id = this.Class._ID++;
            this._super(attributes);

            this._subscriptions = {
                change:[],
                periodic:[],
                archive:[],
                user:[]
            };
        },
        /**
         * Reads this attribute value through the proxy
         *
         * @param cbk
         * @asynchronous
         */
        read:function(cbk) {
            this.proxy.readAttribute(this.name, {timeout:this.timeout}, cbk);
        },
        /**
         * Writes a new value to this attribute through the proxy
         *
         * @param {*} argin
         * @param {Object|Function} cbk
         * @asynchronous
         */
        write:function(argin, cbk) {
            if (this.isReadOnly) return;
            var params = {
                argin: argin,
                timeout:this.timeout
            };
            this.proxy.writeAttribute(this.name, params, cbk);
        },
        poll_id: 0,
        is_polling: false,
        /**
         * Starts continuous polling
         *
         * @param cbk
         * @param interval
         */
        start_polling:function(cbk, interval) {
            console.debug("Starting polling of " + this.element_id());
            this.poll_id = setInterval(
            MVC.Function.bind(function () {
                if (this.is_polling)
                    this.read(cbk)
                }, this),
                interval);
            this.is_polling = true;
        },
        /**
         * Pauses polling
         */
        pause_polling:function(){
            console.debug("Pausing polling of " + this.element_id());
            this.is_polling = false;
        },
        /**
         * Resumes polling
         */
        resume_polling:function(){
            console.debug("Resuming polling of " + this.element_id());
            this.is_polling = true;
        },
        /**
         * Cancels polling. After this method to resume polling one must call start_polling
         */
        stop_polling:function() {
            console.debug("Canceling polling of " + this.element_id());
            clearInterval(this.poll_id);
            this.is_polling = false;
        },
        /**
         * Subscribes to change event of this attribute
         *
         * @param {Function} cbk onEvent
         * @asynchronous
         */
        change:function(cbk){
            var id = this.proxy.subscribe(this.name,mtango.EventTypeEnum.CHANGE,cbk,this.timeout);
            this._subscriptions.change.push(id);
        },
        /**
         * Subscribes to periodic event of this attribute
         *
         * @param {Function} cbk onEvent
         * @asynchronous
         */
        periodic:function(cbk){
            var id = this.proxy.subscribe(this.name,mtango.EventTypeEnum.PERIODIC,cbk,this.timeout);
            this._subscriptions.periodic.push(id);
        },
        /**
         * Subscribes to archive event of this attribute
         *
         * @param {Function} cbk onEvent
         * @asynchronous
         */
        archive:function(cbk){
            var id = this.proxy.subscribe(this.name,mtango.EventTypeEnum.ARCHIVE,cbk,this.timeout);
            this._subscriptions.archive.push(id);
        },
        /**
         * Subscribes to user event of this attribute
         *
         * @param {Function} cbk onEvent
         * @asynchronous
         */
        user:function(cbk){
            var id = this.proxy.subscribe(this.name,mtango.EventTypeEnum.USER,cbk,this.timeout);
            this._subscriptions.user.push(id);
        },
        /**
         * Cancels subscription to change event of this attribute
         */
        cancel_change:function(){
            var me = this;
            this._subscriptions.change.forEach(
                function(id){
                    me.proxy.unsubscribe(me.name,mtango.EventTypeEnum.CHANGE,id);
                }
            );
        },
        /**
         * Cancels subscription to periodic event of this attribute
         */
        cancel_periodic:function(){
            var me = this;
            this._subscriptions.periodic.forEach(
                function(id){
                    me.proxy.unsubscribe(me.name,mtango.EventTypeEnum.CHANGE,id);
                }
            );
        },
        /**
         * Cancels subscription to archive event of this attribute
         */
        cancel_archive:function(){
            var me = this;
            this._subscriptions.archive.forEach(
                function(id){
                    me.proxy.unsubscribe(me.name,mtango.EventTypeEnum.CHANGE,id);
                }
            );
        },
        /**
         * Cancels subscription to user event of this attribute
         */
        cancel_user:function(){
            var me = this;
            this._subscriptions.user.forEach(
                function(id){
                    me.proxy.unsubscribe(me.name,mtango.EventTypeEnum.CHANGE,id);
                }
            );
        },
        /**
         * Cancels all subscriptions of this DeviceAttribute instance
         */
        cancel_all:function(){
            for(var evt in this._subscriptions){
                if(!this._subscriptions.hasOwnProperty(evt)) continue;
                var me = this;
                this._subscriptions[evt].forEach(
                    function(id){
                        me.proxy.unsubscribe(me.name,evt,id);
                    }
                );
            }
        },
        /**
         * {device}/{this.name}, i.e. sys/tg_test/0/double_scalar
         *
         * @returns {string}
         */
        toString:function(){
            return this.proxy.device + '/' + this.name;
        }
    }
);

if(typeof EventTypeEnum == 'undefined') EventTypeEnum = mtango.EventTypeEnum;
if(typeof DeviceAttribute == 'undefined') DeviceAttribute = mtango.DeviceAttribute;;
include.set_path('models');
/**
 * Represents device's command.
 *
 *
 * @type {mtango.DeviceCommand}
 */
mtango.DeviceCommand = MVC.Model.extend('DeviceCommand',
    /* @Static */
    {
        _ID : 1,
        attributes:{
            name: 'string',
            inputType   : 'string',
            inputDescription : 'string',
            outputType  : 'string',
            outputDescription : 'string'
        },
        default_attributes:{
            id: 0
        },
        /**
         * Creates an instance of this model.
         *
         * A JsonP call is executed to obtain information about the command from the remote tango device
         *
         * @param {string} device -- e.g. 'sys/tg_test/1'
         * @param {string} cmd_name -- command's name
         * @param {Object|Function} cbks -- callback(s)
         * @asynchronous
         */
        create:function(device, cmd_name, cbks){
            var callbacks = this._clean_callbacks(cbks);

            var proxy = mtango.DeviceProxy.find(device);
            if (proxy == null) throw "Can not create DeviceCommand! No proxy was found for device: " + device;
            new MVC.JsonP(proxy.full_url + cmd_name + "/info", {
                onComplete: MVC.Function.bind(function (response) {
                    var attributes = {
                        id: this._ID++,
                        proxy: proxy
                    };
                    MVC.Object.extend(attributes, response);
                    var instance = this.create_as_existing(attributes);
                    if (instance.valid())
                        callbacks.onComplete(instance);
                    else
                        callbacks.onFailure(instance);
                }, this)
            });
        }
    },
    /* @Prototype */
    {
        timeout : 5000,
        proxy: null,
        response : null,
        /**
         * Sets this command request timeout
         *
         * @param {int} v
         */
        set_timeout:function(v){
            this.timeout = v;
        },
        /**
         * Sets this command proxy
         *
         * @param {mtango.DeviceProxy} v
         */
        set_proxy:function(v){
            this.proxy = v;
        },
        /**
         * Sets this command response
         *
         * @param {mtango.Response} v
         */
        set_response:function(v){
            this.response = v;
        },
        /**
         * Returns this command element id, i.e. sys/tg_test/1/DevString_1
         *
         * @returns {string}
         */
        element_id:function(){
            return this.proxy.device+'/'+this.name+'_'+this.id;
        },
        /**
         *
         * @param attributes
         * @constructor
         */
        init:function(attributes){
            if(!attributes.id) attributes.id = this.Class._ID++;
            this._super(attributes);
        },
        /**
         * Executes this command on the remote tango device
         *
         * @param {*} argin
         * @param {Object|Function} cbk -- callback(s)
         */
        execute:function(argin, cbk){
            if(typeof argin == 'function' || argin == 'Object') {
                cbk = argin;
                argin = null;
            }
            this.proxy.executeCommand(this.name, {argin:argin},cbk);
        },
        /**
         * @return {string}
         */
        toString:function(){
            return this.proxy.device + '/' + this.name;
        }
    }
);

if(typeof DeviceCommand == 'undefined') DeviceCommand = mtango.DeviceCommand;;
include.end_of_production();